@@ -5,6 +5,7 @@
 
 #include "actionclasses.h"
 #include "amarok.h"
+#include "amarokconfig.h"
 #include "app.h"
 #include "debug.h"
 #include "collectiondb.h"
@@ -300,6 +301,15 @@ AnalyzerContainer::mousePressEvent( QMouseEvent *e)
         AmarokConfig::setCurrentPlaylistAnalyzer( AmarokConfig::currentPlaylistAnalyzer() + 1 );
         changeAnalyzer();
     }
+    else if( e->button() == Qt::RightButton ) {
+        #if defined HAVE_XMMS || defined HAVE_LIBVISUAL
+        KPopupMenu menu;
+        menu.insertItem( SmallIconSet( "visualizations" ), i18n("&Visualizations"), Menu::ID_SHOW_VIS_SELECTOR );
+
+        if( menu.exec( mapToGlobal( e->pos() ) ) == Menu::ID_SHOW_VIS_SELECTOR )
+            Menu::instance()->slotActivated( Menu::ID_SHOW_VIS_SELECTOR );
+        #endif
+    }
 }
 
 
@@ -318,12 +328,11 @@ VolumeAction::plug( QWidget *w, int index )
 {
     //NOTE we only support one plugging currently
 
-    delete (amaroK::Slider*) m_slider; //just in case, remember, we only support one plugging!
+    delete (amaroK::VolumeSlider*) m_slider; //just in case, remember, we only support one plugging!
 
-    m_slider = new amaroK::Slider( Qt::Vertical, w, amaroK::VOLUME_MAX );
+    m_slider = new amaroK::VolumeSlider( w, amaroK::VOLUME_MAX );
     m_slider->setName( "ToolBarVolume" );
     m_slider->setValue( AmarokConfig::masterVolume() );
-    m_slider->setMinimumHeight( 35 );
     m_slider->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Ignored );
 
     QToolTip::add( m_slider, i18n( "Volume control" ) );
@@ -461,8 +470,7 @@ BurnMenu::slotActivated( int index )
 
 StopAction::StopAction( KActionCollection *ac )
   : KAction( i18n( "Stop" ), "player_stop", 0, EngineController::instance(), SLOT( stop() ), ac, "stop" )
-{
-}
+{}
 
 int
 StopAction::plug( QWidget *w, int index )
@@ -483,6 +491,7 @@ StopAction::plug( QWidget *w, int index )
         button->setDelayedPopup( amaroK::StopMenu::instance() );
         button->setName( "toolbutton_stop_menu" );
         button->setIcon( "player_stop" );
+        button->setEnabled( false ); // Required, otherwise the button is always enabled at startup
 
         return containerCount() - 1;
     }
