@@ -23,7 +23,6 @@ email                : markey@web.de
 #include "configdialog.h"
 #include "debug.h"
 #include "collectionbrowser.h"
-
 #include "effectwidget.h"
 #include "enginebase.h"
 #include "enginecontroller.h"
@@ -40,6 +39,7 @@ email                : markey@web.de
 #include "scrobbler.h"
 #include "statusbar.h"
 #include "systray.h"
+#include "threadweaver.h"
 #include "tracktooltip.h"        //engineNewMetaData()
 
 #include <iostream>
@@ -64,8 +64,17 @@ email                : markey@web.de
 #include <qpalette.h>            //applyColorScheme()
 #include <qpixmap.h>             //QPixmap::setDefaultOptimization()
 #include <qpopupmenu.h>          //genericEventHandler
+#include <qtimer.h>              //showHyperThreadingWarning()
 #include <qtooltip.h>            //default tooltip for trayicon
 
+// For the HyperThreading fix
+#ifdef __linux__
+    #ifdef SCHEDAFFINITY_SUPPORT
+        #include <errno.h>
+        #include <sched.h>
+    #endif //SCHEDAFFINITY_SUPPORT
+#endif //__linux__
+
 
 App::App()
         : KApplication()
@@ -81,6 +90,8 @@ App::App()
     //needs to be created before the wizard
     new amaroK::DcopPlayerHandler(); // Must be created first
     new amaroK::DcopPlaylistHandler();
+    new amaroK::DcopPlaylistBrowserHandler();
+    new amaroK::DcopContextBrowserHandler();
     new amaroK::DcopCollectionHandler();
     new amaroK::DcopScriptHandler();
 
@@ -94,6 +105,8 @@ App::App()
         amaroK::config()->sync();
     }
 
+    fixHyperThreading();
+
     m_pGlobalAccel    = new KGlobalAccel( this );
     m_pPlaylistWindow = new PlaylistWindow();
     m_pTray           = new amaroK::TrayIcon( m_pPlaylistWindow );
@@ -143,7 +156,6 @@ App::App()
     // Refetch covers every 80 days or delete every 90 days to comply with Amazon license
     #ifdef AMAZON_SUPPORT
     new RefreshImages();
-    pruneCoverImages();
     #endif
 
     // Trigger collection scan if folder setup was changed by wizard
@@ -183,6 +195,9 @@ App::~App()
     // do even if trayicon is not shown, it is safe
     amaroK::config()->writeEntry( "HiddenOnExit", mainWindow()->isHidden() );
 
+    CollectionDB::instance()->stopScan();
+    ThreadWeaver::deleteInstance(); //waits for jobs to finish
+
     delete m_pPlayerWindow;   //sets some XT keys
     delete m_pPlaylistWindow; //sets some XT keys
 
@@ -201,8 +216,16 @@ App::~App()
 
 void App::handleCliArgs() //static
 {
+    static char cwd[PATH_MAX];
     KCmdLineArgs* const args = KCmdLineArgs::parsedArgs();
 
+    if ( args->isSet( "cwd" ) )
+    {
+        strncpy(cwd, args->getOption( "cwd" ), sizeof(cwd) );
+        cwd[sizeof(cwd)-1] = '\0';
+        KCmdLineArgs::setCwd( cwd );
+    }
+
     if ( args->count() > 0 )
     {
         KURL::List list;
@@ -280,6 +303,7 @@ void App::initCliArgs( int argc, char *argv[] ) //static
             { "toggle-playlist-window", I18N_NOOP("Toggle the Playlist-window"), 0 },
             { "wizard", I18N_NOOP( "Run first-run wizard" ), 0 },
             { "engine <name>", I18N_NOOP( "Use the <name> engine" ), 0 },
+            { "cwd <directory>", I18N_NOOP( "Base for relative filenames/URLs" ), 0 },
             { 0, 0, 0 }
         };
 
@@ -347,6 +371,78 @@ void App::initGlobalShortcuts()
     }
 }
 
+
+void App::fixHyperThreading()
+{
+    // Workaround for stability issues with HyperThreading CPU's, @see BUG 99199.
+    // First we detect the presence of HyperThreading. If active, we bind amarokapp
+    // to the first CPU only (hard affinity).
+    //
+    // @see http://www-128.ibm.com/developerworks/linux/library/l-affinity.html
+    // @see http://www.linuxjournal.com/article/6799
+    // (articles on processor affinity with the linux kernel)
+
+    DEBUG_BLOCK
+
+    #ifdef __linux__
+    QString line;
+    uint cpuCount = 0;
+    QFile cpuinfo( "/proc/cpuinfo" );
+    if ( cpuinfo.open( IO_ReadOnly ) ) {
+        while ( cpuinfo.readLine( line, 20000 ) != -1 ) {
+            if ( line.startsWith( "flags" ) ) {
+                const QString flagsLine = line.section( ":", 1 );
+                const QStringList flags = QStringList::split( " ", flagsLine );
+                if ( flags.contains( "ht" ) ) ++cpuCount;
+            }
+        }
+    }
+    // If multiple CPUs are listed with the HT flag, we got HyperThreading enabled
+    if ( cpuCount > 1 ) {
+        debug() << "CPU with active HyperThreading detected. Enabling WORKAROUND.\n";
+
+        // If the library is new enough try and call sched_setaffinity.
+        #ifdef SCHEDAFFINITY_SUPPORT
+        cpu_set_t mask;
+        CPU_ZERO( &mask ); // Initializes all the bits in the mask to zero
+        CPU_SET( 0, &mask ); // Sets only the bit corresponding to cpu
+        #ifdef SCHEDAFFINITY_3PARAMS
+        if ( sched_setaffinity( 0, sizeof(mask), &mask ) == -1 )
+        #else  //SCHEDAFFINITY_3PARAMS
+        if ( sched_setaffinity( 0, &mask ) == -1 )
+        #endif //SCHEDAFFINITY_3PARAMS
+        {
+            warning() << "sched_setaffinity() call failed with error code: " << errno << endl;
+            QTimer::singleShot( 0, this, SLOT( showHyperThreadingWarning() ) );
+            return;
+        }
+        #else //SCHEDAFFINITY_SUPPORT
+             warning()<<"glibc failed checks for sched_setaffinity" << endl;
+        QTimer::singleShot( 0, this, SLOT( showHyperThreadingWarning() ) );
+        #endif //SCHEDAFFINITY_SUPPORT
+    }
+    else { debug() << "Fix not enabled" << endl; }
+    #else //__linux__
+    debug() << "SCHEDAFFINITY_SUPPORT disabled since this isn't Linux" << endl;
+    #endif //__linux__
+}
+
+
+void App::showHyperThreadingWarning() // SLOT
+{
+    const QString text =
+        i18n( "<p>You are using a processor with the <i>HyperThreading</i> "
+              "feature enabled. Please note that amaroK may be unstable with this "
+              "configuration.</p>"
+              "<p>If you are experiencing problems, use the Linux kernel option 'NOHT', "
+              "or disable <i>HyperThreading</i> in your BIOS setup.</p>"
+              "<p>More information can be found in the README file. For further assistance "
+              "join us at #amarok on irc.freenode.net.</p>" );
+
+    KMessageBox::information( 0, text, i18n( "Warning" ), "showHyperThreadingWarning" );
+}
+
+
 /////////////////////////////////////////////////////////////////////////////////////
 // METHODS
 /////////////////////////////////////////////////////////////////////////////////////
@@ -369,7 +465,7 @@ class ID3v1StringHandler : public TagLib::ID3v1::StringHandler
 
     virtual TagLib::ByteVector render( const TagLib::String &ts ) const
     {
-        const QCString qcs = m_codec->fromUnicode( ts.toCString( true ) );
+        const QCString qcs = m_codec->fromUnicode( TStringToQString(ts) );
         return TagLib::ByteVector( qcs, qcs.length() );
     }
 
@@ -380,6 +476,13 @@ public:
         debug() << "codec: " << m_codec << endl;
         debug() << "codec-name: " << m_codec->name() << endl;
     }
+
+    ID3v1StringHandler( QTextCodec *codec )
+            : m_codec( codec )
+    {
+        debug() << "codec: " << m_codec << endl;
+        debug() << "codec-name: " << m_codec->name() << endl;
+    }
 };
 
 //SLOT
@@ -392,6 +495,8 @@ void App::applySettings( bool firstTime )
     //determine and apply colors first
     applyColorScheme();
 
+    TrackToolTip::instance()->removeFromWidget( m_pTray );
+
     if( AmarokConfig::showPlayerWindow() )
     {
         if( !m_pPlayerWindow )
@@ -414,6 +519,11 @@ void App::applySettings( bool firstTime )
             //KWin::setSystemTrayWindowFor( m_pTray->winId(), m_pPlayerWindow->winId() );
 
             delete m_pTray; m_pTray = new amaroK::TrayIcon( m_pPlayerWindow );
+
+            //make tray icon behave properly after selecting to show or hide player window
+            m_pTray->engineStateChanged(EngineController::instance()->engine()->state(), EngineController::instance()->engine()->state());
+            m_pTray->engineNewMetaData(EngineController::instance()->bundle(), false);
+
         }
         else
             //this is called in the PlayerWindow ctor, hence the else
@@ -422,9 +532,17 @@ void App::applySettings( bool firstTime )
     } else if( m_pPlayerWindow ) {
 
         delete m_pTray; m_pTray = new amaroK::TrayIcon( m_pPlaylistWindow );
+        m_pTray->engineStateChanged(EngineController::instance()->engine()->state(), EngineController::instance()->engine()->state());
+        m_pTray->engineNewMetaData(EngineController::instance()->bundle(), false);
         delete m_pPlayerWindow; m_pPlayerWindow = 0;
 
-        m_pPlaylistWindow->setCaption( "amaroK" );
+        //Set the caption correctly.
+        if ( !EngineController::instance()->bundle().prettyTitle().isEmpty() )
+            m_pPlaylistWindow->setCaption( "amaroK - " + EngineController::instance()->bundle().veryNiceTitle() );
+        else
+            m_pPlaylistWindow->setCaption( "amaroK" );
+
+
         //m_pPlaylistWindow->show(); //must be shown //we do below now
 
         //ensure that at least one Menu is plugged into an accessible UI element
@@ -439,12 +557,17 @@ void App::applySettings( bool firstTime )
     CollectionDB::instance()->applySettings();
     amaroK::StatusBar::instance()->setShown( AmarokConfig::showStatusBar() );
     m_pTray->setShown( AmarokConfig::showTrayIcon() );
+    TrackToolTip::instance()->addToWidget( m_pTray );
 
 
     // we check > 0 because textCodecForIndex( 0 ) crashes amaroK for unknown
     // reasons, also now we assign index 0 to "" in the config combobox
-    if( AmarokConfig::recodeID3v1Tags() && AmarokConfig::recodeEncoding() > 0 )
+    if( AmarokConfig::recodeID3v1Tags() )
+    if( AmarokConfig::recodeEncoding() > 0 )
         TagLib::ID3v1::Tag::setStringHandler( new ID3v1StringHandler( AmarokConfig::recodeEncoding() ) );
+    else
+        // otherwise the locale encoding is used
+        TagLib::ID3v1::Tag::setStringHandler( new ID3v1StringHandler( QTextCodec::codecForLocale() ) );
 
 
     //on startup we need to show the window, but only if it wasn't hidden on exit
@@ -672,7 +795,7 @@ bool amaroK::genericEventHandler( QWidget *recipient, QEvent *e )
             break;
         }
         default:
-            EngineController::instance()->increaseVolume( e->delta() / 18 );
+            EngineController::instance()->increaseVolume( e->delta() / amaroK::VOLUME_SENSITIVITY );
         }
 
         e->accept();
@@ -716,7 +839,7 @@ void App::engineStateChanged( Engine::State state, Engine::State oldState )
         if ( AmarokConfig::showPlayerWindow() )
             m_pPlaylistWindow->setCaption( kapp->makeStdCaption( i18n("Playlist") ) );
         else m_pPlaylistWindow->setCaption( "amaroK" );
-        QToolTip::add( m_pTray, i18n( "amaroK - rediscover your music" ) );
+        TrackToolTip::instance()->clear();
         break;
 
     case Engine::Playing:
@@ -741,19 +864,18 @@ void App::engineStateChanged( Engine::State state, Engine::State oldState )
     }
 }
 
-
 void App::engineNewMetaData( const MetaBundle &bundle, bool /*trackChanged*/ )
 {
     amaroK::OSD::instance()->show( bundle );
     if ( !bundle.prettyTitle().isEmpty() )
         m_pPlaylistWindow->setCaption( "amaroK - " + bundle.veryNiceTitle() );
 
-    TrackToolTip::add( m_pTray, bundle, EngineController::instance()->engine()->position() );
+    TrackToolTip::instance()->setTrack( bundle );
 }
 
 void App::engineTrackPositionChanged( long position, bool /*userSeek*/ )
 {
-    TrackToolTip::add( m_pTray, EngineController::instance()->bundle(), position );
+    TrackToolTip::instance()->setPos( position );
 }
 
 void App::engineVolumeChanged( int newVolume )
@@ -865,44 +987,6 @@ void App::firstRunWizard()
     }
 }
 
-
-void App::pruneCoverImages()
-{
-#ifdef AMAZON_SUPPORT
-    // TODO Offer the option to refresh the images, instead of deleting
-
-    const int MAX_DAYS = 90;
-
-    QDir covers( amaroK::saveLocation( "albumcovers/large/" ) );
-    QDir coverCache( amaroK::saveLocation( "albumcovers/cache/" ) );
-
-    QFileInfoList list( *covers.entryInfoList( QDir::Files ) );
-    QFileInfoList listCache( *coverCache.entryInfoList( QDir::Files ) );
-
-    // Merge both lists
-    for ( uint i = 0; i < listCache.count(); i++ )
-        list.append( listCache.at( i ) );
-
-    // Prune files
-    uint count = 0;
-    uint pruneCount = 0;
-    const QDate currentDate = QDate::currentDate();
-    for ( uint i = 0; i < list.count(); ++i, ++count )
-        if ( list.at( i )->created().date().daysTo( currentDate ) > MAX_DAYS ) {
-            pruneCount++;
-            QFile::remove( list.at( i )->absFilePath() );
-        }
-
-    if ( pruneCount )
-        amaroK::StatusBar::instance()->longMessage( i18n(
-                "amaroK has had to delete some of the images that you downloaded from Amazon. "
-                "Amazon's licensing policy require us to remove content within 90 days of its download. "
-                "You can read more about Amazon's policies "
-                "<a href='http://www.amazon.com/webservices'>here</a>. "
-                "Please note that this only affects covers downloaded using the Cover Manager's fetch covers feature and the covers downloaded using the context menu in the Context Browser, not manually added covers." ) );
-#endif
-}
-
 QWidget *App::mainWindow() const
 {
    return AmarokConfig::showPlayerWindow() ? (QWidget*)m_pPlayerWindow : (QWidget*)m_pPlaylistWindow;
