@@ -37,7 +37,6 @@ class PlaylistWindow;
 class App : public KApplication, public EngineObserver
 {
     Q_OBJECT
-
     public:
         App();
        ~App();
@@ -49,13 +48,15 @@ class App : public KApplication, public EngineObserver
 
         // FRIENDS ------
         friend class PlaylistWindow; //requires access to applySettings()
-
     protected: /* for OSD, tray, and dcop */
         void engineStateChanged( Engine::State state, Engine::State oldState = Engine::Empty );
         void engineNewMetaData( const MetaBundle &bundle, bool trackChanged );
         void engineTrackPositionChanged( long position, bool /*userSeek*/ );
         void engineVolumeChanged( int );
 
+    private slots:
+        void showHyperThreadingWarning();
+
     public slots:
         void applySettings( bool firstTime = false );
         void slotConfigAmarok( const QCString& page = QCString() );
@@ -66,15 +67,15 @@ class App : public KApplication, public EngineObserver
         void firstRunWizard();
 
     private:
+        /** Workaround for HyperThreading CPU's, @see BUG 99199 */
+        void fixHyperThreading();
+
         void initGlobalShortcuts();
         void applyColorScheme();
 
         /** returns the leading window, either playerWindow or playlistWindow */
         QWidget *mainWindow() const;
 
-        /** Removes all covers images older than 90 days, to comply with amazon.com license */
-        void pruneCoverImages();
-
         // ATTRIBUTES ------
         KGlobalAccel        *m_pGlobalAccel;
         PlayerWidget        *m_pPlayerWindow;
