@@ -10,6 +10,7 @@
 #include "config.h"
 #include "coverfetcher.h"
 #include "covermanager.h"
+#include "pixmapviewer.h"
 
 #include <qdesktopwidget.h>  //ctor: desktop size
 #include <qfile.h>
@@ -136,6 +137,7 @@ CoverManager::CoverManager()
     // amazon locale menu
     m_amazonLocaleMenu = new KPopupMenu( this );
     m_amazonLocaleMenu->insertItem( i18n("International"), International );
+    m_amazonLocaleMenu->insertItem( i18n("Canada"), Canada );
     m_amazonLocaleMenu->insertItem( i18n("France"), France );
     m_amazonLocaleMenu->insertItem( i18n("Germany"), Germany );
     m_amazonLocaleMenu->insertItem( i18n("Japan"), Japan);
@@ -157,6 +159,7 @@ CoverManager::CoverManager()
     else if( locale == "de" ) m_currentLocale = Germany;
     else if( locale == "jp" ) m_currentLocale = Japan;
     else if( locale == "uk" ) m_currentLocale = UK;
+    else if( locale == "ca" ) m_currentLocale = Canada;
     else {
         // make sure we handle old config files correctly
         locale = "us";
@@ -249,23 +252,26 @@ void CoverManager::init()
 }
 
 
+CoverViewDialog::CoverViewDialog(const QString& artist, const QString& album, QWidget *parent)
+    : QDialog(parent, 0, false, WDestructiveClose | WType_TopLevel | WNoAutoErase)
+    , m_pixmap(CollectionDB::instance()->albumImage( artist, album, 0 ))
+{
+    KWin::setType( winId(), NET::Utility );
+    kapp->setTopWidget( this );
+    setCaption( kapp->makeStdCaption( artist + i18n(" - ") + album ) );
+
+    m_layout = new QHBoxLayout( this );
+    m_layout->setAutoAdd(true);
+    m_pixmapViewer = new PixmapViewer( this, m_pixmap );
+
+    setFixedSize( m_pixmapViewer->maximalSize() );
+}
+
+
 void CoverManager::viewCover( const QString& artist, const QString& album, QWidget *parent ) //static
 {
     //QDialog means "escape" works as expected
-    QDialog *dialog = new QDialog( parent, 0, false, WDestructiveClose | WType_TopLevel );
-    kapp->setTopWidget( dialog );
-    dialog->setCaption( kapp->makeStdCaption( artist + i18n(" - ") + album ) );
-
-    // Gives the window a small title bar
-    KWin::setType( dialog->winId(), NET::Utility );
-
-    QPixmap pixmap( CollectionDB::instance()->albumImage( artist, album, 0 ) );
-    QLabel* label = new QLabel( dialog );
-    label->setFrameShape( QFrame::NoFrame );
-    dialog->setFixedSize( pixmap.size() );
-    label->setFixedSize( pixmap.size() );
-    label->setPixmap( pixmap );
-
+    QDialog *dialog = new CoverViewDialog( artist, album, parent );
     dialog->show();
 }
 
@@ -278,6 +284,8 @@ QString CoverManager::amazonTld() //static
         return "co.jp";
             else if (AmarokConfig::amazonLocale() == "uk")
         return "co.uk";
+            else if (AmarokConfig::amazonLocale() == "ca")
+        return "ca";
             else
         return AmarokConfig::amazonLocale();
 }
@@ -352,6 +360,12 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
     m_coverView->clear();
     m_coverItems.clear();
 
+    // reset current view mode state to "AllAlbum" which is the default on artist change in left panel
+    m_currentView = AllAlbums;
+    m_viewMenu->setItemChecked( AllAlbums, true );
+    m_viewMenu->setItemChecked( AlbumsWithCover, false );
+    m_viewMenu->setItemChecked( AlbumsWithoutCover, false );
+
     QProgressDialog progress( this, 0, true );
     progress.setLabelText( i18n("Loading Thumbnails...") );
     progress.QDialog::setCaption( i18n("...") );
@@ -599,6 +613,9 @@ void CoverManager::changeLocale( int id ) //SLOT
         case International:
             AmarokConfig::setAmazonLocale( "us" );
             break;
+        case Canada:
+            AmarokConfig::setAmazonLocale( "ca" );
+            break;
         case France:
             AmarokConfig::setAmazonLocale( "fr" );
             break;
@@ -782,8 +799,8 @@ void CoverManager::updateStatusBar()
             if ( values.count() >= 2 )
             {
                 text = i18n( "Fetching cover for " );
-		if ( !values[0].isEmpty() ) text += values[0] + i18n(" - ");
-		text += values[1] + i18n("...");
+        if ( !values[0].isEmpty() ) text += values[0] + i18n(" - ");
+        text += values[1] + i18n("...");
             }
         }
         else if( m_fetchingCovers ) {
@@ -791,7 +808,7 @@ void CoverManager::updateStatusBar()
             if( m_coversFetched )
                 text += i18n( "1 fetched", "%n fetched", m_coversFetched );
             if( m_coverErrors ) {
-		    if( m_coversFetched ) text += i18n(" - ");
+            if( m_coversFetched ) text += i18n(" - ");
                 text += i18n( "1 not found", "%n not found", m_coverErrors );
             }
             if( m_coversFetched + m_coverErrors == 0 )
@@ -975,14 +992,14 @@ void CoverViewItem::paintItem(QPainter* p, const QColorGroup& cg)
     int nameWidth = fm.width( str );
     if( nameWidth > textRect().width() )
     {
-	    QString nameJustify = i18n("...");
+        QString nameJustify = i18n("...");
         int i = 0;
         while ( fm.width( nameJustify + str[ i ] ) < textRect().width() )
             nameJustify += str[ i++ ];
         nameJustify.remove( 0, 3 );
         if ( nameJustify.isEmpty() )
             nameJustify = str.left( 1 );
-	nameJustify += i18n("...");
+    nameJustify += i18n("...");
         str = nameJustify;
     }
     p->setPen( cg.text() );
