@@ -38,14 +38,22 @@
 
 #include "dllpath.h"
 
+#include <config.h>
+
 #include "helix-sp.h"
 #include "hspvoladvise.h"
 #include "utils.h"
-#ifndef TEST_APP
 #include "hsphook.h"
-#endif
 #include "hxfiles.h"
 
+#ifdef USE_HELIX_ALSA
+#include <alsa/asoundlib.h>
+#endif
+
+#ifdef HX_LOG_SUBSYSTEM
+#include "hxtlogutil.h"
+#endif
+
 #ifdef __FreeBSD__
 #define PTHREAD_MUTEX_FAST_NP PTHREAD_MUTEX_NORMAL
 #endif
@@ -62,7 +70,7 @@ class HelixSimplePlayerAudioStreamInfoResponse : public IHXAudioStreamInfoRespon
 {
 public:
    HelixSimplePlayerAudioStreamInfoResponse(HelixSimplePlayer *player, int playerIndex) : 
-      m_Player(player), m_index(playerIndex) {}
+      m_Player(player), m_index(playerIndex), m_lRefCount(0) {}
    virtual ~HelixSimplePlayerAudioStreamInfoResponse() {}
 
    /*
@@ -134,9 +142,9 @@ STDMETHODIMP HelixSimplePlayerAudioStreamInfoResponse::OnStream(IHXAudioStream *
    m_Player->ppctrl[m_index]->pStream = pAudioStream;
    m_Player->ppctrl[m_index]->pPreMixHook = new HSPPreMixAudioHook(m_Player, m_index, pAudioStream);
 
-#ifndef TEST_APP
+   // addpremixhook adds another ref
    pAudioStream->AddPreMixHook(m_Player->ppctrl[m_index]->pPreMixHook, false);
-#endif
+   m_Player->ppctrl[m_index]->pPreMixHook->Release(); // release the addref from the premix constructor
    m_Player->ppctrl[m_index]->bStarting = false;
 
    return HXR_OK;
@@ -148,12 +156,6 @@ const int DEFAULT_STOP_TIME =  -1;
 const int SLEEP_TIME         = 10;
 const int GUID_LEN           = 64;
 
-#ifdef TEST_APP
-// Function prototypes
-void  PrintUsage(const char* pszAppName);
-char* GetAppName(char* pszArgv0);
-#endif
-
 // *** IUnknown methods ***
 
 /////////////////////////////////////////////////////////////////////////
@@ -212,9 +214,8 @@ STDMETHODIMP_(ULONG32) HelixSimplePlayerVolumeAdvice::Release()
     return 0;
 }
 
-STDMETHODIMP HelixSimplePlayerVolumeAdvice::OnVolumeChange(const UINT16 uVolume)
+STDMETHODIMP HelixSimplePlayerVolumeAdvice::OnVolumeChange(const UINT16 /*uVolume*/)
 {
-   m_Player->STDERR("Volume change: %d\n", uVolume);
    m_Player->onVolumeChange(m_index);
 #ifdef HELIX_SW_VOLUME_INTERFACE
    m_Player->ppctrl[m_index]->volume = uVolume;
@@ -224,25 +225,23 @@ STDMETHODIMP HelixSimplePlayerVolumeAdvice::OnVolumeChange(const UINT16 uVolume)
 
 STDMETHODIMP HelixSimplePlayerVolumeAdvice::OnMuteChange(const BOOL bMute)
 {
-   m_Player->STDERR("Mute change: %d\n", bMute);
    m_Player->onMuteChange(m_index);
    m_Player->ppctrl[m_index]->ismute = bMute;
    return HXR_OK;
 }
 
 
-void HelixSimplePlayer::cleanUpStream(int playerIndex)
+void HelixSimplePlayer::cleanUpStream(int /* playerIndex */)
 {
-   ppctrl[playerIndex]->pPreMixHook->Release();
+   // dont do anything here for now
 }
 
 
 void HelixSimplePlayer::updateEQgains()
 {
-#ifndef TEST_APP
    for (int i = 0; i<nNumPlayers; i++)
-      ((HSPPostMixAudioHook *)ppctrl[i]->pPostMixHook)->updateEQgains(m_preamp, m_equalizerGains);
-#endif
+      if (ppctrl[i]->pPostMixHook && isEQenabled())
+         ((HSPPostMixAudioHook *)ppctrl[i]->pPostMixHook)->updateEQgains(m_preamp, m_equalizerGains);
 }
 
 /*
@@ -287,7 +286,7 @@ char* HelixSimplePlayer::RemoveWrappingQuotes(char* str)
    if (len > 0)
    {
       if (str[len-1] == '"') str[--len] = 0;
-      if (str[0] == '"') {int i = 0; do { str[i++] = str[i+1]; } while(--len); }
+      if (str[0] == '"') {int i = 0; do { str[i] = str[i+1]; ++i; } while(--len); }
    }
    return str;
 }
@@ -323,8 +322,19 @@ HelixSimplePlayer::HelixSimplePlayer() :
    scopecount(0),
    scopebufhead(0),
    scopebuftail(0),
+   m_outputsink(OSS),
+   m_device(0),
+#ifdef USE_HELIX_ALSA
+   m_direct(ALSA),  // TODO: out why my alsa direct HW reader doesnt pickup changes in kmix (the whole purpose of this!)
+#else
    m_direct(OSS),
+#endif
+   m_AlsaCapableCore(false),
    m_nDevID(-1),
+   //m_pAlsaPCMHandle(NULL),
+   m_pAlsaMixerHandle(NULL),
+   m_pAlsaMixerElem(NULL),
+   m_alsaDevice("default"),
    m_urlchanged(0),
    m_volBefore(-1),
    m_volAtStart(-1),
@@ -459,9 +469,13 @@ void HelixSimplePlayer::init(const char *corelibhome, const char *pluginslibhome
       STDERR("no engine setup interface\n");
    else
    {
-      pEngineContext = new HSPEngineContext(pCommonClassFactory);
+      pEngineContext = new HSPEngineContext(this, pCommonClassFactory);
       pEngineContext->AddRef();
+#ifdef HX_LOG_SUBSYSTEM
+      HX_ENABLE_LOGGING(pEngineContext);
+#endif
       pEngineSetup->Setup(pEngineContext);
+      pEngineSetup->Release();
    }
 
    // get the client engine selector
@@ -503,9 +517,15 @@ void HelixSimplePlayer::init(const char *corelibhome, const char *pluginslibhome
 
       pPluginProps = 0;
       int n = pPlugin2Handler->GetNumOfPlugins2();
+      m_numPlugins = n;
       STDERR("Got the plugin2 handler: numplugins =  %d\n", n);
-      for (int i=0; i<n; i++)
+      m_pluginInfo = new pluginInfo* [n];
+      for (i=0; i<n; i++)
       {
+         m_pluginInfo[i] = new pluginInfo;
+         m_pluginInfo[i]->description = "";
+         m_pluginInfo[i]->copyright = "";
+         m_pluginInfo[i]->moreinfourl = "";
          hasmime = hasexts = false;
          pPlugin2Handler->GetPluginInfo(i, pPluginProps);
          if (pPluginProps)
@@ -513,20 +533,38 @@ void HelixSimplePlayer::init(const char *corelibhome, const char *pluginslibhome
             ret = pPluginProps->GetFirstPropertyCString(szPropName, pPropValue);
             while(SUCCEEDED(ret))
             {
+               value = (char*)pPropValue->GetBuffer();
                if (!strcmp(szPropName, "FileMime"))
                {
-                  value = (char*)pPropValue->GetBuffer();
                   strcpy(mime, value);
                   hasmime = true;
                }
 
                if (!strcmp(szPropName, "FileExtensions"))
                {
-                  value = (char*)pPropValue->GetBuffer();
                   strcpy(ext, value);
                   hasexts = true;
                }
 
+               if (!strcmp(szPropName, "Description"))
+               {
+                  m_pluginInfo[i]->description = new char[ strlen(value) + 1 ];
+                  strcpy(m_pluginInfo[i]->description, value);
+               }
+
+               if (!strcmp(szPropName, "Copyright"))
+               {
+                  m_pluginInfo[i]->copyright = new char[ strlen(value) + 1 ];
+                  strcpy(m_pluginInfo[i]->copyright, value);
+               }
+
+
+               if (!strcmp(szPropName, "PlgCopy"))
+               {
+                  m_pluginInfo[i]->moreinfourl = new char[ strlen(value) + 1 ];
+                  strcpy(m_pluginInfo[i]->moreinfourl, value);
+               }
+
                ret = pPluginProps->GetNextPropertyCString(szPropName, pPropValue);
             }
             HX_RELEASE(pPluginProps);
@@ -540,12 +578,36 @@ void HelixSimplePlayer::init(const char *corelibhome, const char *pluginslibhome
          }
       }
    }
-
-   openAudioDevice();
-   m_volBefore = m_volAtStart = getDirectHWVolume();
-   STDERR("***VolAtStart is %d\n", m_volAtStart);
 }
 
+int HelixSimplePlayer::initDirectSS()
+{
+   if (m_outputsink == ALSA && m_AlsaCapableCore)
+   {
+      closeAudioDevice();
+      m_direct = ALSA;
+      openAudioDevice();
+   }
+   else if (m_outputsink == ALSA)
+   {
+      closeAudioDevice();
+      m_direct = OSS;
+      m_outputsink = OSS;
+      openAudioDevice();
+      return -1;
+   }
+   else
+   {
+      closeAudioDevice();
+      m_direct = OSS;
+      openAudioDevice();
+   }
+
+   m_volBefore = m_volAtStart = getDirectHWVolume();
+   STDERR("***VolAtStart is %d\n", m_volAtStart);
+   setDirectHWVolume(m_volAtStart);
+   return 0;
+}
 
 int HelixSimplePlayer::addPlayer()
 {
@@ -639,22 +701,21 @@ int HelixSimplePlayer::addPlayer()
    {
       // ...and now the volume interface
       //ppctrl[nNumPlayers]->pVolume = ppctrl[nNumPlayers]->pAudioPlayer->GetAudioVolume();
-      // let's get the Device Volume (like realplayer/helixplayer does)
-      ppctrl[nNumPlayers]->pVolume = ppctrl[nNumPlayers]->pAudioPlayer->GetDeviceVolume();
-      if (!ppctrl[nNumPlayers]->pVolume)
-         STDERR("No Volume Interface - how can we play music!!\n");
-      else
+      //ppctrl[nNumPlayers]->pVolume = ppctrl[nNumPlayers]->pAudioPlayer->GetDeviceVolume();
+      if (ppctrl[nNumPlayers]->pVolume)
       {
+#ifndef HELIX_SW_VOLUME_INTERFACE
          HelixSimplePlayerVolumeAdvice *pVA = new HelixSimplePlayerVolumeAdvice(this, nNumPlayers);
          pVA->AddRef();
          ppctrl[nNumPlayers]->pVolume->AddAdviseSink((IHXVolumeAdviseSink *)pVA);
          ppctrl[nNumPlayers]->pVolumeAdvise = pVA;
-#ifndef HELIX_SW_VOLUME_INTERFACE
          ppctrl[nNumPlayers]->volume = 50; // should get volume advise, which will set this properly
 #else
          // set the helix sw interface volume to 100, we'll control the volume either ourselves post equalization or by
          // amaorok using direct hardware volume
+         //ppctrl[nNumPlayers]->pVolume->SetVolume(100);
          ppctrl[nNumPlayers]->pVolume->SetVolume(100);
+         ppctrl[nNumPlayers]->pVolumeAdvise = 0;
 #endif
       }
 
@@ -663,12 +724,10 @@ int HelixSimplePlayer::addPlayer()
       ppctrl[nNumPlayers]->pAudioPlayer->SetStreamInfoResponse(pASIR);
       ppctrl[nNumPlayers]->pStreamInfoResponse = pASIR;
 
-#ifndef TEST_APP
       // add the post-mix hook (for the scope and the equalizer)
       HSPPostMixAudioHook *pPMAH = new HSPPostMixAudioHook(this, nNumPlayers);
       ppctrl[nNumPlayers]->pAudioPlayer->AddPostMixHook(pPMAH, false /* DisableWrite */, true /* final hook */);
       ppctrl[nNumPlayers]->pPostMixHook = pPMAH;
-#endif
 
       // ...and get the CrossFader
       ppctrl[nNumPlayers]->pAudioPlayer->QueryInterface(IID_IHXAudioCrossFade, (void **) &(ppctrl[nNumPlayers]->pCrossFader));
@@ -700,6 +759,8 @@ void HelixSimplePlayer::tearDown()
    // make sure all players are stopped,
    stop();
 
+   STDERR("TEARDOWN\n");
+
    for (i=nNumPlayers-1; i>=0; i--)
    {
       if (ppctrl[i]->pCrossFader)
@@ -707,16 +768,19 @@ void HelixSimplePlayer::tearDown()
 
       if (ppctrl[i]->pAudioPlayer)
       {
+         
          ppctrl[i]->pAudioPlayer->RemovePostMixHook((IHXAudioHook *)ppctrl[i]->pPostMixHook);
          ppctrl[i]->pPostMixHook->Release(); 
 
          ppctrl[i]->pAudioPlayer->RemoveStreamInfoResponse((IHXAudioStreamInfoResponse *) ppctrl[i]->pStreamInfoResponse);
-         ppctrl[i]->pStreamInfoResponse->Release();
 
-         if (ppctrl[i]->pVolume && ppctrl[i]->pVolumeAdvise)
+         if (ppctrl[i]->pVolume)
          {
-            ppctrl[i]->pVolume->RemoveAdviseSink(ppctrl[i]->pVolumeAdvise);
-            ppctrl[i]->pVolumeAdvise->Release();
+            if (ppctrl[i]->pVolumeAdvise)
+            {
+               ppctrl[i]->pVolume->RemoveAdviseSink(ppctrl[i]->pVolumeAdvise);
+               ppctrl[i]->pVolumeAdvise->Release();
+            }
             ppctrl[i]->pVolume->Release();
          }
 
@@ -779,7 +843,11 @@ void HelixSimplePlayer::tearDown()
    {
       delete [] m_pszGUIDList;
    }
-   
+
+   for (i=0; i<m_numPlugins; i++)
+      delete m_pluginInfo[i];
+   delete [] m_pluginInfo;
+
    if (bEnableVerboseMode)
    {
       STDOUT("\nDone.\n");
@@ -795,7 +863,7 @@ void HelixSimplePlayer::tearDown()
 
    closeAudioDevice();
    
-   theErr = HXR_OK;
+   theErr = HXR_FAILED;
    pErrorSink = NULL;
    pErrorSinkControl = NULL;
    pPluginE = 0;
@@ -823,8 +891,40 @@ void HelixSimplePlayer::tearDown()
    scopebufhead = 0;
    scopebuftail = 0;
    m_preamp = 0;
+
+   delete [] m_device;
 }
 
+
+void HelixSimplePlayer::setOutputSink( HelixSimplePlayer::AUDIOAPI out ) 
+{ 
+#ifdef USE_HELIX_ALSA
+   m_outputsink = out;
+#else
+   m_outputsink = OSS;
+#endif
+}
+
+HelixSimplePlayer::AUDIOAPI HelixSimplePlayer::getOutputSink() 
+{ 
+   return m_outputsink; 
+}
+
+void HelixSimplePlayer::setDevice( const char *dev ) 
+{ 
+   delete [] m_device;
+
+   int len = strlen(dev);
+   m_device = new char [len + 1];
+   strcpy(m_device, dev); 
+}
+
+const char *HelixSimplePlayer::getDevice() 
+{ 
+   return m_device; 
+}
+
+
 #define MAX_DEV_NAME 255
 #define HX_VOLUME  SOUND_MIXER_PCM
 void HelixSimplePlayer::openAudioDevice()
@@ -863,6 +963,88 @@ void HelixSimplePlayer::openAudioDevice()
       }
       break;
 
+      case ALSA:
+      {
+#ifdef USE_HELIX_ALSA
+         int err;
+         
+         STDERR("Opening ALSA mixer device PCM\n");
+
+         err = snd_mixer_open(&m_pAlsaMixerHandle, 0);
+         if (err < 0)
+            STDERR("snd_mixer_open: %s\n", snd_strerror (err));
+  
+         if (err == 0)
+         {
+            err = snd_mixer_attach(m_pAlsaMixerHandle, m_alsaDevice);
+            if (err < 0) 
+               STDERR("snd_mixer_attach: %s\n", snd_strerror (err));
+         }
+
+         if (err == 0)
+         {
+            err = snd_mixer_selem_register(m_pAlsaMixerHandle, NULL, NULL);
+            if (err < 0) 
+               STDERR("snd_mixer_selem_register: %s\n", snd_strerror (err));
+         }
+
+         if (err == 0)
+         {
+            err = snd_mixer_load(m_pAlsaMixerHandle);
+            if(err < 0 )
+               STDERR("snd_mixer_load: %s\n", snd_strerror (err));
+         }
+
+         if (err == 0)
+         {
+            /* Find the mixer element */
+            snd_mixer_elem_t* elem = snd_mixer_first_elem(m_pAlsaMixerHandle);
+            snd_mixer_elem_type_t type;
+            const char* elem_name = NULL;
+            snd_mixer_selem_id_t *sid = NULL;
+
+            snd_mixer_selem_id_alloca(&sid);
+
+            while (elem)
+            {
+               type = snd_mixer_elem_get_type(elem);
+               if (type == SND_MIXER_ELEM_SIMPLE)
+               {
+                  snd_mixer_selem_get_id(elem, sid);
+
+                  /* We're only interested in playback volume controls */
+                  if(snd_mixer_selem_has_playback_volume(elem) && !snd_mixer_selem_has_common_volume(elem) )
+                  {
+                     elem_name = snd_mixer_selem_id_get_name(sid);
+                     if (strcmp(elem_name, "PCM") == 0) // take the first PCM element we find
+                        break;
+                  }
+               }
+            
+               elem = snd_mixer_elem_next(elem);
+            }
+
+            if (!elem)
+            {
+               STDERR("Could not find a usable mixer element\n", snd_strerror(err));
+               err = -1;
+            }
+            
+            m_pAlsaMixerElem = elem;
+         }
+         
+         if (err != 0)
+         {
+            if(m_pAlsaMixerHandle)
+            {
+               snd_mixer_close(m_pAlsaMixerHandle);
+               m_pAlsaMixerHandle = NULL;
+            }
+         }
+#endif
+      }
+      break;
+
       default:
          STDERR("Unknown audio interface in openAudioDevice()\n");
    }
@@ -875,7 +1057,38 @@ void HelixSimplePlayer::closeAudioDevice()
       case OSS:
       {
          if( m_nDevID >= 0 )
+         {
             ::close( m_nDevID );
+            m_nDevID = -1;
+         }
+      }
+      break;
+
+      case ALSA:
+      {
+#ifdef USE_HELIX_ALSA
+         int err;
+
+         if (m_pAlsaMixerHandle && m_pAlsaMixerElem)
+         {
+            err = snd_mixer_detach(m_pAlsaMixerHandle, "PCM");
+            if (err < 0)
+               STDERR("snd_mixer_detach: %s\n", snd_strerror(err));
+  
+            if(err == 0)
+            {
+               err = snd_mixer_close(m_pAlsaMixerHandle);
+               if(err < 0)
+                  STDERR("snd_mixer_close: %s\n", snd_strerror (err));            
+            }
+            
+            if(err == 0)
+            {
+               m_pAlsaMixerHandle = NULL;
+               m_pAlsaMixerElem = NULL;
+            }
+         }
+#endif
       }
       break;
 
@@ -898,14 +1111,68 @@ int HelixSimplePlayer::getDirectHWVolume()
     
          if (m_nDevID < 0 || (::ioctl( m_nDevID, MIXER_READ(HX_VOLUME), &nVolume) < 0))
          {
-            STDERR("ioctl fails when reading HW volume\n");
-            nRetVolume = -1;
+            STDERR("ioctl fails when reading HW volume: mnDevID=%d, errno=%d\n", m_nDevID, errno);
+            nRetVolume = 50; // sensible default
          }
-         nLeftVolume  = (nVolume & 0x000000ff); 
-         nRightVolume = (nVolume & 0x0000ff00) >> 8;
+         else
+         {
+            nLeftVolume  = (nVolume & 0x000000ff); 
+            nRightVolume = (nVolume & 0x0000ff00) >> 8;
          
-         //Which one to use? Average them?
-         nRetVolume = nLeftVolume ;
+            //Which one to use? Average them?
+            nRetVolume = nLeftVolume ;
+         }
+      }
+      break;
+
+      case ALSA:
+      {
+#ifdef USE_HELIX_ALSA
+         if (!m_pAlsaMixerElem)
+            return nRetVolume;
+
+         snd_mixer_elem_type_t type;    
+         int err = 0;
+         type = snd_mixer_elem_get_type(m_pAlsaMixerElem);
+            
+         if (type == SND_MIXER_ELEM_SIMPLE)
+         {
+            long volumeL, volumeR, min_volume, max_volume; 
+
+            if(snd_mixer_selem_has_playback_volume(m_pAlsaMixerElem) || 
+               snd_mixer_selem_has_playback_volume_joined(m_pAlsaMixerElem))
+            {
+               err = snd_mixer_selem_get_playback_volume(m_pAlsaMixerElem,
+                                                         SND_MIXER_SCHN_FRONT_LEFT, 
+                                                         &volumeL);
+               if (err < 0)
+                  STDERR("snd_mixer_selem_get_playback_volume (L): %s\n", snd_strerror (err));
+               else
+               {
+                  if ( snd_mixer_selem_is_playback_mono ( m_pAlsaMixerElem )) 
+                     volumeR = volumeL;
+                  else
+                  {
+                     err = snd_mixer_selem_get_playback_volume(m_pAlsaMixerElem,
+                                                               SND_MIXER_SCHN_FRONT_RIGHT, 
+                                                               &volumeR);
+                     if (err < 0)
+                        STDERR("snd_mixer_selem_get_playback_volume (R): %s\n", snd_strerror (err));
+                  }
+               }
+                  
+               if (err == 0)
+               {
+                  snd_mixer_selem_get_playback_volume_range(m_pAlsaMixerElem,
+                                                            &min_volume,
+                                                            &max_volume);
+
+                  if(max_volume > min_volume)
+                     nRetVolume = (UINT16) (0.5 + (100.0 * (double)(volumeL + volumeR) / (2.0 * (max_volume - min_volume))));
+               }
+            }        
+         }
+#endif
       }
       break;
 
@@ -932,6 +1199,53 @@ void HelixSimplePlayer::setDirectHWVolume(int vol)
       }
       break;
 
+      case ALSA:
+      {
+#ifdef USE_HELIX_ALSA
+         if (!m_pAlsaMixerElem)
+            return;
+
+         snd_mixer_elem_type_t type;    
+         int err = 0;
+         type = snd_mixer_elem_get_type(m_pAlsaMixerElem);
+
+         
+         if (type == SND_MIXER_ELEM_SIMPLE)
+         {
+            long volume, min_volume, max_volume, range; 
+            
+            if(snd_mixer_selem_has_playback_volume(m_pAlsaMixerElem) || 
+               snd_mixer_selem_has_playback_volume_joined(m_pAlsaMixerElem))
+            {
+               snd_mixer_selem_get_playback_volume_range(m_pAlsaMixerElem,
+                                                         &min_volume,
+                                                         &max_volume);
+               
+               range = max_volume - min_volume;
+               volume = (long) (((double)vol / 100) * range + min_volume);
+
+
+               err = snd_mixer_selem_set_playback_volume( m_pAlsaMixerElem,
+                                                          SND_MIXER_SCHN_FRONT_LEFT, 
+                                                          volume);            
+               if (err < 0)
+                  STDERR("snd_mixer_selem_set_playback_volume: %s\n", snd_strerror (err));
+
+               if (!snd_mixer_selem_is_playback_mono (m_pAlsaMixerElem))
+               {
+                  /* Set the right channel too */
+                  err = snd_mixer_selem_set_playback_volume( m_pAlsaMixerElem,
+                                                             SND_MIXER_SCHN_FRONT_RIGHT, 
+                                                             volume);            
+                  if (err < 0)
+                     STDERR("snd_mixer_selem_set_playback_volume: %s\n", snd_strerror (err));
+               }
+            }        
+         }    
+#endif
+      }
+      break;
+
       default:
          STDERR("Unknown audio interface in setDirectHWVolume()\n");
    }
@@ -953,8 +1267,6 @@ int HelixSimplePlayer::setURL(const char *file, int playerIndex)
       if (len >= MAXPATHLEN)
          return -1;;
 
-      //STDERR("Trying to play: %s\n", file);
-      
       if (ppctrl[playerIndex]->pszURL)
          delete [] ppctrl[playerIndex]->pszURL;
       
@@ -1028,21 +1340,14 @@ int HelixSimplePlayer::numPlugins() const
 int HelixSimplePlayer::getPluginInfo(int index, 
                                      const char *&description, 
                                      const char *&copyright, 
-                                     const char *&moreinfourl, 
-                                     unsigned long &ver) const
+                                     const char *&moreinfourl) const
 {
-   IHXPlugin *p = 0;
-   HXBOOL mult;
-
-   description = "";
-   copyright = "";
-   moreinfourl = "";
-   pPluginE->GetPlugin((ULONG32)index, (IUnknown *&)p);
-   if (p)
+   if (index < m_numPlugins)
    {
-      p->AddRef();
-      p->GetPluginInfo(mult, description, copyright, moreinfourl, (ULONG32 &) ver);
-      p->Release();
+      description = m_pluginInfo[index]->description;
+      copyright   = m_pluginInfo[index]->copyright;
+      moreinfourl = m_pluginInfo[index]->moreinfourl;
+   
       return 0;
    }
    return -1;
@@ -1384,7 +1689,8 @@ void HelixSimplePlayer::seek(unsigned long pos, int playerIndex)
 unsigned long HelixSimplePlayer::where(int playerIndex) const
 {
    if (playerIndex < nNumPlayers && ppctrl[playerIndex]->pHSPContext)
-      return ppctrl[playerIndex]->pHSPContext->position();
+      //return ppctrl[playerIndex]->pHSPContext->position();
+      return ppctrl[playerIndex]->pPlayer->GetCurrentPlayTime();
    else
       return 0;
 }
@@ -1401,11 +1707,12 @@ unsigned long HelixSimplePlayer::getVolume(int playerIndex)
 {
    unsigned long vol;
 
-   if (playerIndex < nNumPlayers && ppctrl[playerIndex]->pVolume)
+   if (playerIndex < nNumPlayers)
    {
       pthread_mutex_lock(&m_engine_m);
       vol = ppctrl[playerIndex]->volume;
-//pVolume->GetVolume();
+      //if (ppctrl[playerIndex]->pVolume)
+         //pVolume->GetVolume();
       pthread_mutex_unlock(&m_engine_m);
 
       return (vol);
@@ -1429,9 +1736,7 @@ void HelixSimplePlayer::setVolume(unsigned long vol, int playerIndex)
          pthread_mutex_lock(&m_engine_m);
 #ifndef HELIX_SW_VOLUME_INTERFACE
          ppctrl[playerIndex]->volume = vol;
-#ifndef TEST_APP
          ((HSPPostMixAudioHook *)ppctrl[playerIndex]->pPostMixHook)->setGain(vol);
-#endif
 #else
          ppctrl[playerIndex]->pVolume->SetVolume(vol);
 #endif
@@ -1567,324 +1872,3 @@ void HelixSimplePlayer::clearScopeQ()
       delete item;
 }
 
-
-
-#ifdef TEST_APP
-char* GetAppName(char* pszArgv0)
-{
-    char* pszAppName;
-
-    pszAppName = strrchr(pszArgv0, '\\');
-
-    if (NULL == pszAppName)
-    {
-        return pszArgv0;
-    }
-    else
-    {
-        return pszAppName + 1;
-    }
-}
-
-void PrintUsage(const char* pszAppName)
-{
-    STDOUT("\n");
-
-#if defined _DEBUG || defined DEBUG
-    STDOUT("USAGE:\n%s [-as0] [-d D] [-n N] [-t T] [-st ST] [-g file] [-u username] [-p password] <URL>\n", pszAppName);
-#else
-    STDOUT("USAGE:\n%s [-as0] [-n N] [-t T] [-g file] [-u username] [-p password] <URL>\n", pszAppName);
-#endif
-    STDOUT("       -a : optional flag to show advise sink output\n");
-    STDOUT("       -s : optional flag to output useful status messages\n");
-    STDOUT("       -0 : optional flag to disable all output windows\n");
-    STDOUT("       -l : optional flag to tell the player where to find its DLLs\n");
-
-#if defined _DEBUG || defined DEBUG
-    STDOUT("       -d : HEX flag to print out DEBUG info\n");
-    STDOUT("            0x8000 -- for audio methods calling sequence\n"
-            "0x0002 -- for variable values\n");
-#endif
-    STDOUT("       -n : optional flag to spawn N players\n");
-    STDOUT("       -rn: optional flag to repeat playback N times\n");
-    STDOUT("       -t : wait T milliseconds between starting players (default: %d)\n", DEFAULT_TIME_DELTA);
-    STDOUT("       -st: wait ST milliseconds util stoping players (default: %d)\n", DEFAULT_STOP_TIME);
-    STDOUT("       -g : use the list of GUIDS in the specified newline-delimited file\n");
-    STDOUT("            to give each of the N players a different GUID\n");
-    STDOUT("       -u : username to use in authentication response\n");
-    STDOUT("       -p : password to use in authentication response\n");
-    STDOUT("\n");
-}
-
-#include <pthread.h>
-
-struct pt_t
-{
-   int playerIndex;
-   HelixSimplePlayer *splay;
-};
-
-void *play_thread(void *arg)
-{
-   struct pt_t *pt = (struct pt_t *) arg;
-   
-   STDERR("play_thread started for player %d\n",pt->playerIndex);
-   pt->splay->play(pt->playerIndex);
-}
-
-
-int main( int argc, char *argv[] )
-{
-   int   i;
-   char  dllhome[MAX_PATH];
-   int   nNumPlayers = 1;
-   int   nNumPlayRepeats = 1;
-   int   nTimeDelta = DEFAULT_TIME_DELTA;
-   int   nStopTime = DEFAULT_STOP_TIME;
-   bool  bURLFound = false;
-
-    //See if the user has set their HELIX_LIBS env var. This is overridden by the
-    //-l option.
-    const char* pszHelixLibs = getenv("HELIX_LIBS");
-    if( pszHelixLibs )
-        SafeStrCpy( dllhome,  pszHelixLibs, MAX_PATH);
-
-    int volscale = 100;
-
-    for (i = 1; i < argc; i++)
-    {
-       if (0 == stricmp(argv[i], "-v"))
-       {
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -n option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          volscale = atoi(argv[i]);
-       }
-       else if (0 == stricmp(argv[i], "-n"))
-       {
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -n option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          nNumPlayers = atoi(argv[i]);
-          if (nNumPlayers < 1)
-          {
-             STDOUT("\nError: Invalid value for -n option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-       }
-       else if (0 == stricmp(argv[i], "-rn"))
-       {
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -rn option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          nNumPlayRepeats = atoi(argv[i]);
-          if (nNumPlayRepeats < 1)
-          {
-             STDOUT("\nError: Invalid value for -rn option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-       }
-       else if (0 == stricmp(argv[i], "-t"))
-       {
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -t option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          nTimeDelta = atoi(argv[i]);
-          if (nTimeDelta < 0)
-          {
-             STDOUT("\nError: Invalid value for -t option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-       }
-       else if (0 == stricmp(argv[i], "-st"))
-       {
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -st option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          nStopTime = atoi(argv[i]);
-          if (nStopTime < 0)
-          {
-             STDOUT("\nError: Invalid value for -st option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-       }
-#if defined _DEBUG || defined DEBUG
-       else if (0 == stricmp(argv[i], "-d"))
-       {
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -d option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          debug_level() = (int)strtoul(argv[i], 0, 0);
-       }
-#endif
-       else if (0 == stricmp(argv[i], "-u"))
-       {
-          char *puser = new char[1024];
-          strcpy(puser, "");
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -u option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          SafeStrCpy(puser,  argv[i], 1024);
-          //HelixSimplePlayer::setUsername(puser);
-       }
-       else if (0 == stricmp(argv[i], "-p"))
-       {
-          char *ppass = new char[1024];
-          strcpy(ppass, ""); /* Flawfinder: ignore */
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -p option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          SafeStrCpy(ppass,  argv[i], 1024);
-          //HelixSimplePlayer::setPassword(ppass);
-       }
-       else if (0 == stricmp(argv[i], "-g"))
-       {
-          char *pfile = new char[1024];
-          strcpy(pfile, ""); /* Flawfinder: ignore */
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -g option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          SafeStrCpy(pfile, HelixSimplePlayer::RemoveWrappingQuotes(argv[i]), 1024);
-          //HelixSimplePlayer::setGUIDFile(pfile);
-          //if (!HelixSimplePlayer::ReadGUIDFile())
-          //{
-          //   STDOUT("\nError: Unable to read file specified by -g option.\n\n");
-          //   return -1;
-          //}
-       }
-       else if (0 == stricmp(argv[i], "-l"))
-       {
-          if (++i == argc)
-          {
-             STDOUT("\nError: Invalid value for -l option.\n\n");
-             PrintUsage(GetAppName(argv[0]));
-             return -1;
-          }
-          SafeStrCpy(dllhome, HelixSimplePlayer::RemoveWrappingQuotes(argv[i]), MAX_PATH);
-       }
-       else if (!bURLFound)
-       {
-          bURLFound  = true;
-          //if no "://" was found lets add file:// by default so that you
-          //can refer to local content as just ./splay ~/Content/startrek.rm,
-          //for example, and not ./splay file:///home/gregory/Content/startrek.rm
-       }
-       else
-       {
-//            PrintUsage(GetAppName(argv[0]));
-//            return -1;
-       }
-    }
-    
-    if (!bURLFound)
-    {
-       if (argc > 1)
-       {
-          STDOUT("\nError: No media file or URL was specified.\n\n");
-       }
-       PrintUsage(GetAppName(argv[0]));
-       return -1;
-    }
-    // start first player
-    
-    HelixSimplePlayer splay;
-    pthread_t thr1, thr2;
-    struct pt_t x = { 1, &splay };
-
-    splay.init("/usr/local/RealPlayer/common","/usr/local/RealPlayer/plugins","/usr/local/RealPlayer/codecs", 2);
-    
-    pthread_create(&thr1, 0, play_thread, (void *) &x);
-
-    //splay.enableCrossFader(0, 1);
-    splay.setURL(argv[1],1);
-    bool xfstart = 0, didseek = 0, xfsetup = 0;
-    unsigned long d = 15000;
-    unsigned long xfpos;
-    unsigned long counter = 0;
-    int nowplayingon = 0;
-
-    //splay.dispatch();
-    STDERR("Before sleep\n");
-    sleep(2);
-
-    //splay.start(0);
-    while (!splay.done())
-    {
-       //splay.dispatch();
-       sleep(1);
-       if (!(counter % 10))
-          STDERR("time: %ld/%ld %ld/%ld count %d\n", splay.where(0), splay.duration(0), splay.where(1), splay.duration(1), counter);
-       
-       counter++;
-       
-       if (splay.duration(x.playerIndex))
-          xfpos = splay.duration(x.playerIndex) - d;
-
-       if (!xfsetup && splay.where(x.playerIndex) > xfpos - 2000)
-       {
-          xfsetup = 1;
-          //splay.crossFade(argv[i-1], splay.where(splay.xf().fromIndex), d);
-          x.playerIndex = 0;
-          splay.setURL(argv[i-1],1);
-          pthread_create(&thr2, 0, play_thread, (void *) &x);
-       }
-
-       //if (!xfstart && splay.where(splay.xf().fromIndex) > xfpos - 1000)
-       //{
-       //   xfstart = 1;
-       //   splay.startCrossFade();
-       //}
-    }
-
-
-//    if (splay.getError() == HXR_OK)
-//    {
-//       int vol = splay.getVolume(0);
-//       bool mute;
-//
-//       splay.setMute(false);
-//       mute = splay.getMute(0);
-//       if (mute)
-//          STDERR("Mute is ON\n");
-//       else
-//          STDERR("Mute is OFF\n");
-//
-//       splay.setVolume(volscale);
-//       vol = splay.getVolume(0);
-//       splay.play(argv[i-1]);
-//    }
-}
-
-
-#endif // TEST_APP
