@@ -28,7 +28,9 @@
 
 #ifdef HAVE_NMM
 
+#include "debug.h"
 #include "plugin/plugin.h"
+
 #include <nmm/base/graph/GraphBuilder2.hpp>
 #include <nmm/base/registry/NodeDescription.hpp>
 #include <nmm/base/ProxyApplication.hpp>
@@ -41,6 +43,9 @@
 #include <nmm/interfaces/general/ITrackDuration.hpp>
 #include <nmm/interfaces/device/audio/IAudioDevice.hpp>
 #include <nmm/base/ProxyObject.hpp>
+
+#include <qapplication.h>
+
 #include <kfileitem.h>
 #include <kmimetype.h>
 #include <iostream>
@@ -49,20 +54,20 @@
 AMAROK_EXPORT_PLUGIN( NmmEngine )
 
 NmmEngine::NmmEngine()
-  : EngineBase(),
-    __track_length(0),
+  : Engine::Base(),
     __position(0),
+    __track_length(0),
     __state(Engine::Empty),
-    __composite(0),
     __app(0),
     __endTrack_listener(this, &NmmEngine::endTrack),
     __syncReset_listener(this, &NmmEngine::syncReset),
     __setProgress_listener(this, &NmmEngine::setProgress),
     __trackDuration_listener(this, &NmmEngine::trackDuration),
+    __composite(0),
     __playback(0),
     __seeking(false),
-    __running(true),
     __track_ended(false),
+    __running(true),
     __cond(__mutex)
 {
   // start the secondary thread
@@ -356,7 +361,7 @@ void NmmEngine::run()
   while (__running) {
     __cond.wait();
     if (__running) {
-      emit trackEnded();
+      QApplication::postEvent(this, new QCustomEvent(3000) );
     }
   }
 }
@@ -375,6 +380,20 @@ Result NmmEngine::endTrack()
   return SUCCESS;
 }
 
+void NmmEngine::customEvent( QCustomEvent *e )
+{
+    switch( e->type() )
+    {
+        case 3000:
+            // emit signal from GUI thread
+            emit trackEnded();
+            break;
+
+        default:
+            ;
+    }
+}
+
 Result NmmEngine::syncReset()
 {
   __seeking = false;
