@@ -38,7 +38,7 @@
 
 using namespace NMM;
 
-class NmmEngine : public EngineBase, public ThreadedObject
+class NmmEngine : public Engine::Base, public ThreadedObject
 {
 Q_OBJECT
 public:
@@ -94,6 +94,8 @@ private:
      */
     void run();
 
+    void customEvent( QCustomEvent* );
+
     /**
      * The current track position in milliseconds.
      */
