@@ -9,6 +9,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#define DEBUG_PREFIX "xine-engine"
+
 #include "xine-config.h"
 #include "xinecfg.h"
 #include "xine-engine.h"
@@ -47,7 +49,7 @@ namespace Log
     static uint bufferCount = 0;
     static uint scopeCallCount = 1; //prevent divideByZero
     static uint noSuitableBuffer = 0;
-};
+}
 
 ///returns the configuration we will use. there is no KInstance, so using this hacked up method.
 //static inline QCString configPath() { return QFile::encodeName(KStandardDirs().localkdedir() + KStandardDirs::kde_default("data") + "amarok/xine-config"); }
@@ -68,7 +70,9 @@ XineEngine::XineEngine()
     addPluginProperty( "StreamingMode", "NoStreaming" );
     addPluginProperty( "HasConfigure", "true" );
     addPluginProperty( "HasEqualizer", "true" );
+    #ifndef __NetBSD__  // NetBSD does not offer audio mixing
     addPluginProperty( "HasCrossfade", "true" );
+    #endif
 }
 
 XineEngine::~XineEngine()
@@ -201,6 +205,7 @@ XineEngine::load( const KURL &url, bool isStream )
     if( m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
     {
        s_fader = new Fader( this );
+       setEqualizerParameters( m_intPreamp, m_equalizerGains );
     }
 
    // for users who stubbonly refuse to use DMIX or buy a good soundcard
@@ -382,26 +387,16 @@ XineEngine::setEqualizerParameters( int preamp, const QValueList<int> &gains )
     m_equalizerGains = gains;
     m_intPreamp = preamp;
     QValueList<int>::ConstIterator it = gains.begin();
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_30HZ, *it );
     xine_set_param( m_stream, XINE_PARAM_EQ_60HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_125HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_250HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_500HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_1000HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_2000HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_4000HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_8000HZ, *++it );
-    debug() << "eq setting " << *it << endl;
     xine_set_param( m_stream, XINE_PARAM_EQ_16000HZ, *++it );
-    debug() << "eq setting " << *it << endl;
 
     m_preamp = ( preamp / 2 + 150 ) / 100.0;
     setVolume( m_volume );
@@ -410,16 +405,35 @@ XineEngine::setEqualizerParameters( int preamp, const QValueList<int> &gains )
 bool
 XineEngine::canDecode( const KURL &url ) const
 {
-    //TODO should free the file_extensions char*
-    static QStringList list = QStringList::split( ' ', xine_get_file_extensions( m_xine ) );
-
+    static QStringList list;
+    if(list.isEmpty())
+    {
+        char* exts = xine_get_file_extensions( m_xine );
+        list = QStringList::split( ' ', exts );
+        delete [] exts; exts = NULL;
+        //images
+        list.remove("png");
+        list.remove("jpg");
+        list.remove("jpeg");
+        list.remove("gif");
+        list.remove("ilbm");
+        list.remove("iff");
+        //subtitles
+        list.remove("asc");
+        list.remove("txt");
+        list.remove("sub");
+        list.remove("srt");
+        list.remove("smi");
+        list.remove("ssa");
+//HACK we also check for m4a because xine plays them but
+//for some reason doesn't return the extension
+        if(!list.contains("m4a"))
+            list << "m4a";
+    }
     const QString path = url.path();
     const QString ext  = path.mid( path.findRev( '.' ) + 1 ).lower();
 
-    //HACK we also check for m4a because xine plays them but
-    //     for some reason doesn't return the extension
-
-    return ext != "txt" && (list.contains( ext ) || ext =="m4a");
+    return list.contains( ext );
 }
 
 const Engine::Scope&
@@ -549,8 +563,9 @@ XineEngine::customEvent( QCustomEvent *e )
 
     case 3003: { //meta info has changed
 
-        Engine::SimpleMetaBundle bundle;
+        debug() << "Metadata received." << endl;
 
+        Engine::SimpleMetaBundle bundle;
         bundle.title   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_TITLE ) );
         bundle.artist  = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ARTIST ) );
         bundle.album   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ALBUM ) );
