@@ -22,6 +22,7 @@
 #include <qpainter.h>
 #include <qregexp.h>
 #include <qsimplerichtext.h>
+#include <qtimer.h>
 #include <qtooltip.h>       //QToolTip::add()
 #include <qfileinfo.h>
 #include <qdir.h>
@@ -293,7 +294,7 @@ MediaDeviceList::viewportPaintEvent( QPaintEvent *e )
 
     // Superimpose bubble help:
 
-    if ( childCount() == 0 )
+    if ( !m_parent->m_connectButton->isOn() )
     {
         QPainter p( viewport() );
 
@@ -420,7 +421,7 @@ MediaDeviceView::MediaDeviceView( MediaBrowser* parent )
     QToolTip::add( m_configButton,   i18n( "Configure mount commands" ) );
 
     m_connectButton->setToggleButton( true );
-    m_connectButton->setOn( m_deviceList->childCount() != 0 );
+    m_connectButton->setOn( m_device->isConnected() ||  m_deviceList->childCount() != 0 );
     m_transferButton->setDisabled( true );
 
     m_progress->setFixedHeight( m_transferButton->sizeHint().height() );
@@ -429,6 +430,31 @@ MediaDeviceView::MediaDeviceView( MediaBrowser* parent )
     connect( m_connectButton,  SIGNAL( clicked() ), MediaDevice::instance(), SLOT( ipodConnection() ) );
     connect( m_transferButton, SIGNAL( clicked() ), MediaDevice::instance(), SLOT( transferFiles() ) );
     connect( m_configButton,   SIGNAL( clicked() ), MediaDevice::instance(), SLOT( config() ) );
+    connect( m_transferList, SIGNAL( rightButtonPressed( QListViewItem*, const QPoint&, int ) ),
+             this,   SLOT( slotShowContextMenu( QListViewItem*, const QPoint&, int ) ) );}
+
+void
+MediaDeviceView::slotShowContextMenu( QListViewItem* item, const QPoint& point, int )
+{
+    if ( item )
+    {
+        KPopupMenu menu( this );
+
+        enum Actions { REMOVE_SELECTED, CLEAR_ALL };
+
+        menu.insertItem( SmallIconSet( "edittrash" ), i18n( "&Remove From Queue" ), REMOVE_SELECTED );
+        menu.insertItem( SmallIconSet( "view_remove" ), i18n( "&Clear Queue" ), CLEAR_ALL );
+
+        switch( menu.exec( point ) )
+        {
+            case REMOVE_SELECTED:
+                m_device->removeSelected();
+                break;
+            case CLEAR_ALL:
+                m_device->clearItems();
+                break;
+        }
+    }
 }
 
 
@@ -502,9 +528,9 @@ MediaDevice::addURL( const KURL& url )
 
         m_transferURLs << url;
         m_parent->m_stats->setText( i18n( "1 track in queue", "%n tracks in queue", m_parent->m_transferList->childCount() ) );
-        m_parent->m_transferButton->setDisabled( false );
+        m_parent->m_transferButton->setEnabled( m_parent->m_device->isConnected() || m_parent->m_deviceList->childCount() != 0 );
     } else
-        amaroK::StatusBar::instance()->longMessage( i18n( "Track already exists on iPod: " + url.path().local8Bit() ) );
+        amaroK::StatusBar::instance()->longMessage( i18n( "Track already exists on iPod: " + url.path().local8Bit() ), KDE::StatusBar::Sorry );
 }
 
 void
@@ -512,7 +538,7 @@ MediaDevice::addURLs( const KURL::List urls )
 {
         KURL::List::ConstIterator it = urls.begin();
         for ( ; it != urls.end(); ++it )
-            addURL( (*it).path() );
+            addURL( *it );
 }
 
 QStringList
@@ -605,6 +631,30 @@ MediaDevice::songsByArtistAlbum( const QString& artist, const QString& album )
     return items;
 }
 
+void
+MediaDevice::clearItems()
+{
+    m_parent->m_transferList->clear();
+    m_transferURLs.clear();
+    m_parent->m_stats->setText( i18n( "0 tracks in queue" ) );
+    m_parent->m_transferButton->setEnabled( false );
+}
+
+void
+MediaDevice::removeSelected()
+{
+    QPtrList<QListViewItem>  selected = m_parent->m_transferList->selectedItems();
+
+    for( QListViewItem *item = selected.first(); item; item = selected.next() )
+    {
+        m_transferURLs.remove( item->text( 0 ) );
+        m_parent->m_transferList->takeItem( item );
+        delete item;
+    }
+    m_parent->m_transferButton->setEnabled( m_parent->m_transferList->childCount() != 0 );
+    m_parent->m_stats->setText( i18n( "1 track in queue", "%n tracks in queue", m_parent->m_transferList->childCount() ) );
+}
+
 void
 MediaDevice::config()
 {
@@ -756,7 +806,14 @@ MediaDevice::deleteFiles( const KURL::List& urls )
 {
     //NOTE we assume that currentItem is the main target
     int count  = urls.count();
-    int button = KMessageBox::warningContinueCancel( m_parent->m_parent,
+
+    //Enable after string freeze.
+    //m_parent->m_stats->setText( i18n( "1 track to be deleted", "%n tracks to be deleted", count ) );
+    m_parent->m_progress->setProgress( 0 );
+    m_parent->m_progress->setTotalSteps( count );
+    m_parent->m_progress->show();
+
+    int button = KMessageBox::warningContinueCancel( m_parent,
                                                      i18n( "<p>You have selected 1 file to be <b>irreversibly</b> deleted.",
                                                            "<p>You have selected %n files to be <b>irreversibly</b> deleted.",
                                                            count
@@ -766,10 +823,13 @@ MediaDevice::deleteFiles( const KURL::List& urls )
 
     if ( button == KMessageBox::Continue )
     {
+
         KURL::List::ConstIterator it = urls.begin();
         for ( ; it != urls.end(); ++it )
-            QFile::remove( (*it).path() );
-
+        {
+            m_parent->m_progress->setProgress( m_parent->m_progress->progress() + 1 );
+            KIO::del( *it, false, false );
+        }
         if ( m_ipod->ensureConsistency() )
         {
             m_ipod->lock( true );
@@ -780,6 +840,9 @@ MediaDevice::deleteFiles( const KURL::List& urls )
         else
             debug() << "iPod inconsistent!" << endl;
     }
+    QTimer::singleShot( 1500, m_parent->m_progress, SLOT(hide()) );
+    //Enable after string freeze
+    //m_parent->m_stats->setText( i18n( "1 track in queue", "%n tracks in queue", m_parent->m_transferList->childCount() ) );
 }
 
 
@@ -848,7 +911,8 @@ MediaDevice::deleteFromIPod( MediaItem* item )
 void
 MediaDevice::ipodConnection() //SLOT
 {
-    if ( m_parent->m_connectButton->isOn() ){
+    if ( m_parent->m_connectButton->isOn() )
+    {
         if ( !m_mntcmd.isEmpty() )
         {
             mount();
@@ -857,43 +921,56 @@ MediaDevice::ipodConnection() //SLOT
         openIPod();
         m_parent->m_deviceList->renderView( 0 );
 
-        if( m_parent->m_deviceList->childCount() != 0 )
+        if( isConnected() || m_parent->m_deviceList->childCount() != 0 )
         {
             m_parent->m_connectButton->setOn( true );
+            if ( m_parent->m_transferList->childCount() != 0 )
+            {
+                m_parent->m_transferButton->setEnabled( true );
+                //Enable after string freeze
+                //m_parent->m_stats->setText( i18n( "Checking device for duplicate files." ) );
+                KURL::List urls;
+                urls = m_transferURLs;
+                clearItems();
+                addURLs( urls );
+           }
         }
         else
         {
+            m_parent->m_connectButton->setOn( false );
             KMessageBox::error( m_parent->m_parent,
                 i18n( "Could not find device, please mount it and try again." ),
                 i18n( "Media Device Browser" ) );
-            m_parent->m_connectButton->setOn( false );
         }
     }
-    else{
-    if ( m_parent->m_transferList->childCount() != 0 &&  m_ipod->isStillConnected() )
+    else
+    {
+        if ( m_parent->m_transferList->childCount() != 0 &&  m_ipod->isStillConnected() )
         {
             int button = KMessageBox::warningContinueCancel( m_parent->m_parent,
-                                                        i18n( "There are tracks queued for transfer."
-                                                          " Would you like to transfer them before disconnecting?"),
-                                                         i18n( "Media Device Browser" ),
-                                                         KGuiItem(i18n("&Transfer"),"rebuild") );
+                                                            i18n( "There are tracks queued for transfer."
+                                                            " Would you like to transfer them before disconnecting?"),
+                                                            i18n( "Media Device Browser" ),
+                                                            KGuiItem(i18n("&Transfer"),"rebuild") );
 
-          if ( button == KMessageBox::Continue )
+            if ( button == KMessageBox::Continue )
                 transferFiles();
-      }
+        }
+
         fileTransferFinished();
         m_ipod->close();
-        QString text=i18n( "Your device is now in sync, please unmount it and disconnect now." );
+        QString text = i18n( "Your device is now in sync, please unmount it and disconnect now." );
+
         if ( !m_umntcmd.isEmpty() )
         {
             umount();
             text=i18n( "Your device is now in sync, you can disconnect now." );
         }
+
         m_ipod = new IPod::IPod();
         m_parent->m_deviceList->renderView( 0 );
         m_parent->m_connectButton->setOn( false );
-        KMessageBox::error( m_parent->m_parent,
-            text, i18n( "Media Device Browser" ) );
+        KMessageBox::information( m_parent->m_parent, text, i18n( "Media Device Browser" ) );
     }
 }
 
