@@ -102,6 +102,9 @@ class MediaDeviceView : public QVBox
         MediaDeviceView( MediaBrowser* parent );
         ~MediaDeviceView();
 
+    private slots:
+        void slotShowContextMenu( QListViewItem* item, const QPoint& point, int );
+
     private:
         QLabel*          m_stats;
         KProgress*       m_progress;
@@ -126,26 +129,30 @@ class MediaDevice : public QObject
         MediaDevice( MediaDeviceView* parent );
         ~MediaDevice();
 
-        void addURL( const KURL& url );
-        void addURLs( const KURL::List urls );
+        void        addURL( const KURL& url );
+        void        addURLs( const KURL::List urls );
+        bool        isConnected() { return m_ipod->isStillConnected(); }
         QStringList items( QListViewItem* item );
-        KURL::List songsByArtist( const QString& artist );
-        KURL::List songsByArtistAlbum( const QString& artist, const QString& album );
-        QString m_mntcmd;
-        QString m_umntcmd;
+        KURL::List  songsByArtist( const QString& artist );
+        KURL::List  songsByArtistAlbum( const QString& artist, const QString& album );
+
+        QString     m_mntcmd;
+        QString     m_umntcmd;
 
         static MediaDevice *instance() { return s_instance; }
 
     public slots:
+        void clearItems();
         void config();
-        void setMountCommand(const QString & mnt);
-        void setUmountCommand(const QString & umnt);
-        int mount();
-        int umount();
-        void transferFiles();
         void deleteFiles( const KURL::List& urls );
         void deleteFromIPod( MediaItem* item );
         void ipodConnection();
+        int  mount();
+        void removeSelected();
+        void setMountCommand(const QString & mnt);
+        void setUmountCommand(const QString & umnt);
+        int  umount();
+        void transferFiles();
 
     private slots:
         void fileTransferred();
@@ -153,16 +160,16 @@ class MediaDevice : public QObject
         void syncIPod();
 
     private:
-        int sysCall(const QString & command);
-        void openIPod();
-        bool fileExists( const MetaBundle& bundle );
-        KURL::List m_transferURLs;
-        KShellProcess     *sysProc;
-
-        bool m_wait;
+        int              sysCall(const QString & command);
+        void             openIPod();
+        bool             fileExists( const MetaBundle& bundle );
+        KURL::List       m_transferURLs;
+        KShellProcess   *sysProc;
 
+        bool             m_wait;
         MediaDeviceView* m_parent;
-        IPod::IPod* m_ipod;
+        IPod::IPod*      m_ipod;
+
         static MediaDevice *s_instance;
 };
 
