@@ -40,6 +40,7 @@ email                : markey@web.de
 #include <kapplication.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kpushbutton.h>
 #include <kstandarddirs.h>
 #include <kurldrag.h>
 #include <kwin.h>            //eventFilter()
@@ -51,13 +52,19 @@ email                : markey@web.de
 //simple function for fetching amarok images
 namespace amaroK
 {
+    //TODO remove these, they suck, do a generic getImage
+
     QPixmap getPNG( const QString &filename )
     {
-        return QPixmap( locate( "data", QString( "amarok/images/%1.png" ).arg( filename ) ), "PNG" );
+        QString file = !filename.endsWith( ".png", false ) ? "amarok/images/%1.png" : "amarok/images/%1";
+
+        return QPixmap( locate( "data", file.arg( filename ) ), "PNG" );
     }
 
     QPixmap getJPG( const QString &filename )
     {
+        QString file = !filename.endsWith( ".jpg", false ) ? "amarok/images/%1.jpg" : "amarok/images/%1";
+
         return QPixmap( locate( "data", QString( "amarok/images/%1.jpg" ).arg( filename ) ), "JPEG" );
     }
 }
@@ -76,10 +83,10 @@ createWidget( const QRect &r, QWidget *parent, const char *name = 0, Qt::WFlags
 }
 
 
-
 PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlaylist )
     : QWidget( parent, name, Qt::WType_TopLevel )
     , EngineObserver( EngineController::instance() )
+    , m_minimalView( false )
     , m_pAnimTimer( new QTimer( this ) )
     , m_scrollBuffer( 291, 16 )
     , m_plusPixmap( getPNG( "time_plus" ) )
@@ -126,6 +133,12 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
                          new NavButton( m_pFrameButtons, "stop", ac->action( "stop" ) );
                          new NavButton( m_pFrameButtons, "next", ac->action( "next" ) );
 
+        KPushButton *switchView = new KPushButton( KGuiItem( "", "mini_dock" ), m_pFrameButtons );
+        switchView->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred ); // too big!
+        switchView->setFocusPolicy( QWidget::NoFocus );
+        connect( switchView, SIGNAL( clicked() ), SLOT( toggleView() ) );
+
+
         m_pButtonPlay->setToggleButton( true );
         m_pButtonPause->setToggleButton( true );
     } //</NavButtons>
@@ -198,6 +211,8 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
     //KWin::setOnAllDesktops( winId(), true );
 
     connect( m_pAnimTimer, SIGNAL( timeout() ), SLOT( drawScroll() ) );
+
+    TrackToolTip::instance()->addToWidget( m_pScrollFrame );
 }
 
 
@@ -205,6 +220,7 @@ PlayerWidget::~PlayerWidget()
 {
     AmarokConfig::setPlayerPos( pos() );
     AmarokConfig::setPlaylistWindowEnabled( m_pPlaylistButton->isOn() );
+    TrackToolTip::instance()->removeFromWidget( m_pScrollFrame );
 }
 
 
@@ -214,8 +230,8 @@ void PlayerWidget::setScroll( const QStringList &list )
 {
     QString text;
     QStringList list2( list );
-
-    for( QStringList::Iterator it = list2.begin(); it != list2.end(); )
+    QStringList::Iterator end( list2.end() );
+    for( QStringList::Iterator it = list2.begin(); it != end; )
     {
         if( !(*it).isEmpty() )
         {
@@ -293,7 +309,7 @@ void PlayerWidget::drawScroll()
 }
 
 
-void PlayerWidget::engineStateChanged( Engine::State state )
+void PlayerWidget::engineStateChanged( Engine::State state, Engine::State /*oldState*/ )
 {
     DEBUG_BLOCK
 
@@ -314,8 +330,11 @@ void PlayerWidget::engineStateChanged( Engine::State state )
             break;
 
         case Engine::Playing:
-            m_pTimeLabel->show();
-            m_pTimeSign->show();
+            if( !m_minimalView )
+            {
+                m_pTimeLabel->show();
+                m_pTimeSign->show();
+            }
             m_pButtonPlay->setOn( true );
             m_pButtonPause->setOn( false );
             break;
@@ -357,14 +376,11 @@ void PlayerWidget::engineNewMetaData( const MetaBundle &bundle, bool )
     if( bundle.length() ) list << bundle.prettyLength();
     setScroll( list );
 
-    //update image tooltip
-    TrackToolTip::add( m_pScrollFrame, bundle );
-
     update(); //we need to update rateString
 }
 
 
-void PlayerWidget::engineTrackPositionChanged( long position )
+void PlayerWidget::engineTrackPositionChanged( long position, bool /*userSeek*/ )
 {
     m_pSlider->setValue( position );
 
@@ -459,6 +475,37 @@ void PlayerWidget::applySettings()
     }
 }
 
+void PlayerWidget::setMinimalView( bool enable )
+{
+    m_pAnalyzer->setHidden( enable );
+    m_pTimeLabel->setHidden( enable );
+    m_pTimeSign->setHidden( enable );
+    m_pDescription->setHidden( enable );
+    m_pButtonEq->setHidden( enable );
+    m_pPlaylistButton->setHidden( enable );
+    m_pVolSlider->setHidden( enable );
+
+    if( enable )
+    {
+        uint space = 2;
+        m_pScrollFrame->setGeometry ( 6,space,  m_pScrollFrame->width(),  m_pScrollFrame->height() );
+        m_pSlider->setGeometry      ( 4,space + m_pScrollFrame->height(), 303,12 );
+        m_pFrameButtons->setGeometry( 0,space + m_pScrollFrame->height() + m_pSlider->height(), 311,22 );
+        uint height = m_pFrameButtons->height() + m_pScrollFrame->height() + m_pSlider->height() + space;
+        setFixedSize( 311, height );
+    }
+    else
+    {
+        m_pScrollFrame->setGeometry( 6,18, m_pScrollFrame->width(),m_pScrollFrame->height() );
+        m_pSlider->setGeometry( 4,103, 303,12 );
+        m_pFrameButtons->setGeometry(0, 118, 311,22);
+        setFixedSize( 311, 140 );
+    }
+
+    m_minimalView = enable;
+    update();
+}
+
 
 // EVENTS -----------------------------------------------------------------
 
@@ -474,7 +521,7 @@ bool PlayerWidget::event( QEvent *e )
     case QEvent::Close:
 
         amaroK::genericEventHandler( this, e );
-        return TRUE; //we handled it
+        return true; //we handled it
 
     case QEvent::ApplicationPaletteChange:
 
@@ -483,24 +530,25 @@ bool PlayerWidget::event( QEvent *e )
             determineAmarokColors();
             applySettings();
         }
-        return TRUE;
+        return true;
 
     case 6/*QEvent::KeyPress*/:
         if (static_cast<QKeyEvent*>(e)->key() == Qt::Key_D/* && (m_pAnalyzer->inherits("QGLWidget")*/)
         {
-            if (m_pAnalyzer->parent() != 0)
+            if( m_pAnalyzer->parent() )
             {
-                m_pAnalyzer->reparent(0, QPoint(50,50), true);
+                m_pAnalyzer->reparent( 0, QPoint(50,50), true );
                 m_pAnalyzer->setCaption( kapp->makeStdCaption( i18n("Analyzer") ) );
                 m_pAnalyzer->installEventFilter( this );
                 m_pAnalyzer->setPaletteBackgroundColor( paletteBackgroundColor() );
                 QToolTip::remove( m_pAnalyzer );
             }
-            else createAnalyzer( 0 );
+            else
+                createAnalyzer( 0 );
 
-            return TRUE; //eat event
+            return true; //eat event
         }
-        return FALSE; //don't eat event
+        return false; //don't eat event
 
     case QEvent::Show:
 
@@ -559,7 +607,7 @@ bool PlayerWidget::event( QEvent *e )
             }
         }
 
-        return FALSE;
+        return false;
 
     case QEvent::Hide:
         m_pAnimTimer->stop();
@@ -578,15 +626,6 @@ bool PlayerWidget::event( QEvent *e )
 
                 const KWin::WindowInfo info = KWin::windowInfo( winId() );
 
-                #if KDE_IS_VERSION(3,2,1)
-                if( info.hasState( NET::Shaded ) )
-                {
-                    //FIXME this works if the OSD is up, and only then sometimes
-                    //      I think it's maybe a KWin bug..
-                    parentWidget()->hide();
-                }
-                else
-                #endif
                 if( info.isMinimized() ) KWin::iconifyWindow( parentWidget()->winId(), false );
                 else
                     //this may seem strange, but it is correct
@@ -601,7 +640,7 @@ bool PlayerWidget::event( QEvent *e )
                 parentWidget()->hide();
         }
 
-        return FALSE;
+        return false;
 
     default:
         return QWidget::event( e );
@@ -631,9 +670,9 @@ PlayerWidget::eventFilter( QObject *o, QEvent *e )
         if( e->type() == QEvent::Close )
         {
             createAnalyzer( 0 );
-            return TRUE;
+            return true;
         }
-        return FALSE;
+        return false;
     }
 
     switch( e->type() )
@@ -641,7 +680,7 @@ PlayerWidget::eventFilter( QObject *o, QEvent *e )
     case QEvent::Close:
 
         static_cast<QCloseEvent*>(e)->accept(); //close the window!
-        return TRUE; //don't let PlaylistWindow have the event - see PlaylistWindow::closeEvent()
+        return true; //don't let PlaylistWindow have the event - see PlaylistWindow::closeEvent()
 
     case QEvent::Hide:
 
@@ -661,11 +700,7 @@ PlayerWidget::eventFilter( QObject *o, QEvent *e )
 
             KWin::WindowInfo info = KWin::windowInfo( parentWidget()->winId() );
 
-            if( !(info.isMinimized()
-            #if KDE_IS_VERSION(3,2,1)
-            || info.hasState( NET::Shaded )
-            #endif
-            ) ) break;
+            if( !info.isMinimized() ) break;
         }
 
         //FALL THROUGH
@@ -688,7 +723,7 @@ PlayerWidget::eventFilter( QObject *o, QEvent *e )
         break;
     }
 
-    return FALSE;
+    return false;
 }
 
 
@@ -696,7 +731,8 @@ void PlayerWidget::paintEvent( QPaintEvent* )
 {
     //uses widget's font and foregroundColor() - see ctor
     QPainter p( this );
-    p.drawText( 6, 68, m_rateString );
+    if( !m_minimalView )
+        p.drawText( 6, 68, m_rateString );
 
     bitBlt( m_pScrollFrame, 0, 0, &m_scrollBuffer );
     bitBlt( m_pTimeLabel,   0, 0, &m_timeBuffer );
