@@ -19,6 +19,7 @@
 #include <qpainter.h>          //paintCell()
 #include <qpixmap.h>           //paintCell()
 
+#include <kdeversion.h>        //KDE_VERSION ifndefs.  Remove this once we reach a kde 4 dep
 #include <kiconloader.h>       //smallIcon
 #include <kio/job.h>           //podcast retrieval
 #include <kio/jobclasses.h>    //podcast retrieval
@@ -49,6 +50,44 @@ class PlaylistReader : public ThreadWeaver::DependentJob
         const QString m_path;
 };
 
+/////////////////////////////////////////////////////////////////////////////
+///    CLASS PlaylistBrowserEntry
+////////////////////////////////////////////////////////////////////////////
+
+int
+PlaylistBrowserEntry::compare( QListViewItem* item, int col, bool ascending ) const
+{
+    bool i1 = rtti() == PlaylistCategory::RTTI;
+    bool i2 = item->rtti() == PlaylistCategory::RTTI;
+
+    // If only one of them is a category, make it show up before
+    if ( i1 != i2 )
+        return i1 ? -1 : 1;
+    else if ( i1 ) //both are categories
+    {
+        PlaylistBrowser * const pb = PlaylistBrowser::instance();
+
+        QValueList<PlaylistCategory*> toplevels; //define a static order for the toplevel categories
+        toplevels << pb->m_playlistCategory
+                  << pb->m_smartCategory
+                  << pb->m_dynamicCategory
+                  << pb->m_streamsCategory
+                  << pb->m_podcastCategory;
+
+        for( int i = 0, n = toplevels.count(); i < n; ++i )
+        {
+            if( this == toplevels[i] )
+                return ascending ? -1 : 1; //same order whether or not it's ascending
+            if( item == toplevels[i] )
+                return ascending ? 1 : -1;
+        }
+    }
+
+    return KListViewItem::compare(item, col, ascending);
+}
+
+
+
 /////////////////////////////////////////////////////////////////////////////
 ///    CLASS PlaylistCategory
 ////////////////////////////////////////////////////////////////////////////
@@ -64,7 +103,6 @@ PlaylistCategory::PlaylistCategory( QListView *parent, QListViewItem *after, con
     setPixmap( 0, SmallIcon("folder_red") );
 
     setText( 0, t );
-    setText( 1, "" );
 }
 
 
@@ -79,11 +117,10 @@ PlaylistCategory::PlaylistCategory( PlaylistCategory *parent, QListViewItem *aft
     setPixmap( 0, SmallIcon("folder") );
 
     setText( 0, t );
-    setText( 1, "" );
 }
 
 
-PlaylistCategory::PlaylistCategory( QListView *parent, QListViewItem *after, QDomElement xmlDefinition, bool isFolder )
+PlaylistCategory::PlaylistCategory( QListView *parent, QListViewItem *after, const QDomElement &xmlDefinition, bool isFolder )
     : PlaylistBrowserEntry( parent, after )
     , m_folder( isFolder )
 {
@@ -92,11 +129,10 @@ PlaylistCategory::PlaylistCategory( QListView *parent, QListViewItem *after, QDo
     setRenameEnabled( 0, isFolder );
 
     setPixmap( 0, SmallIcon("folder_red") );
-    setText( 1, "" );
 }
 
 
-PlaylistCategory::PlaylistCategory( PlaylistCategory *parent, QListViewItem *after, QDomElement xmlDefinition )
+PlaylistCategory::PlaylistCategory( PlaylistCategory *parent, QListViewItem *after, const QDomElement &xmlDefinition )
     : PlaylistBrowserEntry( parent, after )
     , m_folder( true )
 {
@@ -108,8 +144,9 @@ PlaylistCategory::PlaylistCategory( PlaylistCategory *parent, QListViewItem *aft
 }
 
 
-void PlaylistCategory::setXml( QDomElement xml )
+void PlaylistCategory::setXml( const QDomElement &xml )
 {
+    PlaylistBrowser *pb = PlaylistBrowser::instance();
     QString tname = xml.tagName();
     if ( tname == "category" ) {
         QListViewItem *last = 0;
@@ -131,6 +168,31 @@ void PlaylistCategory::setXml( QDomElement xml )
             else if ( e.tagName() == "party" ) {
                 last = new PartyEntry( this, last, e );
             }
+            else if ( e.tagName() == "podcast" )
+            {
+                const KURL url( n.namedItem( "url").toElement().text() );
+                QString xmlLocation = amaroK::saveLocation( "podcasts/" );
+                xmlLocation        += n.namedItem( "cache" ).toElement().text();
+
+                QDomDocument xml;
+                QFile xmlFile( xmlLocation );
+                QTextStream stream( &xmlFile );
+                stream.setEncoding( QTextStream::UnicodeUTF8 );
+
+                if( !xmlFile.open( IO_ReadOnly ) || !xml.setContent( stream.read() ) )
+                {
+                    // Invalid podcasts should still be added to the browser, which means there is no cached xml.
+                    last = new PodcastChannel( this, last, url );
+                    continue;
+                }
+
+                last = new PodcastChannel( this, last, url, n, xml );
+
+                #define item static_cast<PodcastChannel*>(last)
+                if( item->autoScan() )
+                    pb->m_podcastItemsToScan.append( item );
+                #undef  item
+            }
         }
         setText( 0, xml.attribute("name") );
     }
@@ -146,7 +208,7 @@ QDomElement PlaylistCategory::xml()
           //FIXME: this is a very ugly and bad hack not to save the default smart and stream lists.
             if ( it->text(0) == i18n("Cool-Streams") || it->text(0) == i18n("Collection") )
                 continue;
-            i.appendChild( it->xml() );
+            i.appendChild( d.importNode( it->xml(), true ) );
         }
         return i;
 }
@@ -200,7 +262,7 @@ PlaylistEntry::PlaylistEntry( QListViewItem *parent, QListViewItem *after, const
 }
 
 
-PlaylistEntry::PlaylistEntry( QListViewItem *parent, QListViewItem *after, QDomElement xmlDefinition )
+PlaylistEntry::PlaylistEntry( QListViewItem *parent, QListViewItem *after, const QDomElement &xmlDefinition )
     : PlaylistBrowserEntry( parent, after )
     , m_loading( false )
     , m_loaded( false )
@@ -248,6 +310,10 @@ void PlaylistEntry::load()
     //starts loading animation
     ((PlaylistBrowserView *)listView())->startAnimation( this );
 
+    //delete all children, so that we don't duplicate things
+    while( firstChild() )
+        delete firstChild();
+
      //read the playlist file in a thread
     ThreadWeaver::instance()->queueJob( new PlaylistReader( this, m_url.path() ) );
 }
@@ -291,7 +357,7 @@ void PlaylistEntry::insertTracks( QListViewItem *after, KURL::List list, QMap<QS
         QString title = str.section(';',0,0);
         int length = str.section(';',1,1).toUInt();
 
-        TrackItemInfo *newInfo = new TrackItemInfo( *it, title.isEmpty() ? key : title, length );
+        TrackItemInfo *newInfo = new TrackItemInfo( *it, title, length );
         m_length += newInfo->length();
         m_trackCount++;
 
@@ -309,14 +375,13 @@ void PlaylistEntry::insertTracks( QListViewItem *after, KURL::List list, QMap<QS
             else
                 tmp_droppedTracks.append( newInfo );
         }
-
     }
 
-    setModified( true );    //show a save icon to save changes
+    PlaylistBrowser::instance()->savePlaylist( this );
 }
 
 
-void PlaylistEntry::removeTrack( QListViewItem *item )
+void PlaylistEntry::removeTrack( QListViewItem *item, bool isLast )
 {
     #define item static_cast<PlaylistTrackItem*>(item)
     //remove a track and update playlist stats
@@ -332,7 +397,8 @@ void PlaylistEntry::removeTrack( QListViewItem *item )
 
     #undef item
 
-    setModified( true );    //show a save icon to save changes
+    if( isLast )
+        PlaylistBrowser::instance()->savePlaylist( this );
 }
 
 
@@ -391,20 +457,14 @@ void PlaylistEntry::setOpen( bool open )
             load();
             return;
         }
-
     }
     else if( !isDynamic() || !m_dynamic ) {    //collapse
 
-        QListViewItem* child = firstChild();
-        QListViewItem* childTmp;
         //delete all children
-        while ( child ) {
-            childTmp = child;
-            child = child->nextSibling();
-            delete childTmp;
-        }
-        m_lastTrack = 0;
+        while( firstChild() )
+            delete firstChild();
 
+        m_lastTrack = 0;
     }
 
     QListViewItem::setOpen( open );
@@ -461,6 +521,7 @@ void PlaylistEntry::setModified( bool chg )
         else {
             delete m_savePix;
             m_savePix = 0;
+            tmp_droppedTracks.clear();
         }
 
         m_modified = chg;
@@ -501,7 +562,11 @@ void PlaylistEntry::paintCell( QPainter *p, const QColorGroup &cg, int column, i
 
     QPainter pBuf( &buffer, true );
     // use alternate background
+#if KDE_VERSION < KDE_MAKE_VERSION(3,3,91)
     pBuf.fillRect( buffer.rect(), isSelected() ? cg.highlight() : backgroundColor() );
+#else
+    pBuf.fillRect( buffer.rect(), isSelected() ? cg.highlight() : backgroundColor(0) );
+#endif
 
     KListView *lv = (KListView *)listView();
 
@@ -643,7 +708,18 @@ TrackItemInfo::TrackItemInfo( const KURL &u, const QString &t, const int l )
         , m_length( l )
 {
     if( m_title.isEmpty() )
-        m_title = MetaBundle::prettyTitle( fileBaseName( m_url.path() ) );
+    {
+        MetaBundle *mb = new MetaBundle( u );
+        if( mb->isValidMedia() )
+        {
+            m_title = mb->prettyTitle();
+            m_length = mb->length();
+        }
+        else
+        {
+            m_title = MetaBundle::prettyTitle( fileBaseName( m_url.path() ) );
+        }
+    }
 
     if( m_length < 0 )
         m_length = 0;
@@ -670,7 +746,7 @@ StreamEntry::StreamEntry( QListViewItem *parent, QListViewItem *after, const KUR
     setText( 0, m_title );
 }
 
-StreamEntry::StreamEntry( QListViewItem *parent, QListViewItem *after, QDomElement xmlDefinition )
+StreamEntry::StreamEntry( QListViewItem *parent, QListViewItem *after, const QDomElement &xmlDefinition )
     : PlaylistBrowserEntry( parent, after )
 {
     setDragEnabled( true );
@@ -731,7 +807,11 @@ void StreamEntry::paintCell( QPainter *p, const QColorGroup &cg, int column, int
 
     QPainter pBuf( &buffer, true );
     // use alternate background
+#if KDE_VERSION < KDE_MAKE_VERSION(3,3,91)
     pBuf.fillRect( buffer.rect(), isSelected() ? cg.highlight() : backgroundColor() );
+#else
+    pBuf.fillRect( buffer.rect(), isSelected() ? cg.highlight() : backgroundColor(0) );
+#endif
 
     KListView *lv = (KListView *)listView();
 
@@ -819,7 +899,7 @@ StreamEditor::StreamEditor( QWidget *parent, const char *name )
 }
 
 // For editing
-StreamEditor::StreamEditor( QWidget *parent, QString title, QString url, const char *name )
+StreamEditor::StreamEditor( QWidget *parent, const QString &title, const QString &url, const char *name )
     : KDialogBase( parent, name, true, i18n("Edit Radio Stream"), Ok|Cancel)
 {
     makeGridMainWidget( 2, Qt::Horizontal );
@@ -859,7 +939,7 @@ PartyEntry::PartyEntry( QListViewItem *parent, QListViewItem *after, const QStri
     setText( 0, name );
 }
 
-PartyEntry::PartyEntry( QListViewItem *parent, QListViewItem *after, QDomElement xmlDefinition )
+PartyEntry::PartyEntry( QListViewItem *parent, QListViewItem *after, const QDomElement &xmlDefinition )
         : PlaylistBrowserEntry( parent, after )
 {
     setPixmap( 0, SmallIcon( "dynamic" ) );
@@ -949,6 +1029,7 @@ QDomElement PartyEntry::xml() {
 PodcastChannel::PodcastChannel( QListViewItem *parent, QListViewItem *after, const KURL &url )
     : PlaylistBrowserEntry( parent, after )
     , m_url( url )
+    , m_saveLocation( amaroK::saveLocation( "podcasts/data/" ) )
     , m_loading1( QPixmap( locate("data", "amarok/images/loading1.png" ) ) )
     , m_loading2( QPixmap( locate("data", "amarok/images/loading2.png" ) ) )
     , m_fetching( false )
@@ -957,10 +1038,9 @@ PodcastChannel::PodcastChannel( QListViewItem *parent, QListViewItem *after, con
     , m_hasProblem( false )
     , m_autoScan( true )
     , m_interval( 4 )
-    , m_mediaFetch( DOWNLOAD )
+    , m_mediaFetch( STREAM )
     , m_purgeItems( false )
     , m_purgeCount( 2 ) // we do a small hack here to make sure we only download the first 2 items of a new pc.
-    , m_first( 0 )
     , m_last( 0 )
 {
     setDragEnabled( true );
@@ -976,9 +1056,10 @@ PodcastChannel::PodcastChannel( QListViewItem *parent, QListViewItem *after, con
 
 
 PodcastChannel::PodcastChannel( QListViewItem *parent, QListViewItem *after,
-                                const KURL &url, QDomNode channelSettings, QDomDocument xmlDefinition )
+                                const KURL &url, const QDomNode &channelSettings, const QDomDocument &xmlDefinition )
     : PlaylistBrowserEntry( parent, after )
     , m_url( url )
+    , m_saveLocation( channelSettings.namedItem( "savelocation").toElement().text() )
     , m_loading1( QPixmap( locate("data", "amarok/images/loading1.png" ) ) )
     , m_loading2( QPixmap( locate("data", "amarok/images/loading2.png" ) ) )
     , m_fetching( false )
@@ -989,15 +1070,21 @@ PodcastChannel::PodcastChannel( QListViewItem *parent, QListViewItem *after,
     , m_interval( channelSettings.namedItem( "scaninterval").toElement().text().toInt() )
     , m_purgeItems( channelSettings.namedItem( "purge").toElement().text() == "true" )
     , m_purgeCount( channelSettings.namedItem( "purgecount").toElement().text().toInt() )
-    , m_first( 0 )
     , m_last( 0 )
 {
-    if( channelSettings.namedItem( "fetch").toElement().text() == "download" )
-        m_mediaFetch = DOWNLOAD;
+    if( channelSettings.namedItem( "fetch").toElement().text() == "automatic" )
+        m_mediaFetch = AUTOMATIC;
     else
         m_mediaFetch = STREAM;
 
-    setXml( xmlDefinition.namedItem("rss").namedItem("channel") );
+    if( m_saveLocation.isEmpty() )
+        m_saveLocation = KURL::fromPathOrURL( amaroK::saveLocation( "podcasts/data/" ) );
+
+    QDomNode type = xmlDefinition.namedItem("rss");
+    if( !type.isNull() )
+        setXml( type.namedItem("channel"), RSS );
+    else
+        setXml( xmlDefinition.namedItem("feed"), ATOM );
 
     setDragEnabled( true );
     setRenameEnabled( 0, false );
@@ -1008,43 +1095,149 @@ PodcastChannel::PodcastChannel( QListViewItem *parent, QListViewItem *after,
 void
 PodcastChannel::configure()
 {
+    // Save the values
     QString url    = m_url.prettyURL();
+    QString save   = m_saveLocation.path();
+    bool autoScan  = m_autoScan;
+    int mediaFetch = m_mediaFetch;
     int purgeCount = m_purgeCount;
 
-    PodcastSettings *settings = new PodcastSettings( m_url, m_autoScan, m_interval,
+    PodcastSettings *settings = new PodcastSettings( url, save, m_autoScan, m_interval,
                                                      m_mediaFetch, m_purgeItems, m_purgeCount );
-
     if( settings->exec() == QDialog::Accepted )
     {
+        m_url        = KURL::fromPathOrURL( settings->url() );
+        save         = settings->saveLocation();
+        m_autoScan   = settings->hasAutoScan();
+        m_interval   = settings->interval();
+        m_mediaFetch = settings->fetch();
+        m_purgeItems = settings->hasPurge();
+        m_purgeCount = settings->purgeCount();
+
+
+        bool downloadMedia = ( (mediaFetch != m_mediaFetch) && (m_mediaFetch == AUTOMATIC) );
+
         if( url != m_url.prettyURL() )
         {
             removeChildren();
             fetch();
         }
-        else if( m_purgeItems && ( m_purgeCount < purgeCount ) ) //fetch() will purge, so no need to fall through
+
+        if( m_purgeItems && ( m_purgeCount < purgeCount ) )
         {
             purge();
         }
+
+        /**
+         * Rewrite local url
+         * Move any downloaded media to the new location
+         */
+        if( save != m_saveLocation.path() )
+        {
+            KURL::List copyList;
+            m_saveLocation = KURL::fromPathOrURL( save );
+
+            PodcastItem *item = static_cast<PodcastItem*>( firstChild() );
+            // get a list of the urls of already downloaded items
+            while( item )
+            {
+                if( item->hasDownloaded() )
+                    copyList << item->localUrl();
+
+                item->setLocalUrlBase( save );
+                item = static_cast<PodcastItem*>( item->nextSibling() );
+            }
+            // move the items
+            if( !copyList.isEmpty() )
+            {
+                KIO::CopyJob* m_podcastMoveJob = new KIO::CopyJob( copyList, m_saveLocation, KIO::CopyJob::Move, false, false );
+                amaroK::StatusBar::instance()->newProgressOperation( m_podcastMoveJob )
+                        .setDescription( i18n( "Moving Podcasts" ) );
+            }
+        }
+
+        if( autoScan != m_autoScan )
+        {
+            if( m_autoScan )
+                PlaylistBrowser::instance()->m_podcastItemsToScan.append( this );
+            else
+                PlaylistBrowser::instance()->m_podcastItemsToScan.remove( this );
+        }
+
+        if( downloadMedia )
+        {
+            downloadChildren();
+        }
+
+        if( settings->applyToAll() )
+            PlaylistBrowser::instance()->setGlobalPodcastSettings( this );
     }
 }
 
+void
+PodcastChannel::downloadChildren()
+{
+    QListViewItem *item = firstChild();
+    while( item )
+    {
+        #define item static_cast<PodcastItem*>(item)
+        if( !item->hasDownloaded() )
+            m_podcastDownloadQueue.append( item );
+        #undef  item
+
+        item = item->nextSibling();
+    }
+    downloadChildQueue();
+}
+
+void
+PodcastChannel::downloadChildQueue()
+{
+    if( m_podcastDownloadQueue.isEmpty() ) return;
+
+    PodcastItem *first = m_podcastDownloadQueue.first();
+    first->downloadMedia();
+    m_podcastDownloadQueue.removeFirst();
+
+    connect( first, SIGNAL( downloadFinished() ), this, SLOT( downloadChildQueue() ) );
+}
+
+const KURL
+PodcastChannel::xmlUrl()
+{
+    return KURL::fromPathOrURL( amaroK::saveLocation( "podcasts/" ) +  m_cache );
+}
+
 void
 PodcastChannel::fetch()
 {
     setText(0, i18n( "Retrieving Podcast..." ) );
 
+    m_iconCounter = 1;
     startAnimation();
     connect( &m_animationTimer, SIGNAL(timeout()), this, SLOT(slotAnimation()) );
 
     m_podcastJob = KIO::storedGet( m_url, false, false );
 
     amaroK::StatusBar::instance()->newProgressOperation( m_podcastJob )
-            .setDescription( i18n( "Fetching Podcast" ) );
+            .setDescription( i18n( "Fetching Podcast" ) )
+            .setAbortSlot( this, SLOT( abortFetch() ) );
 
     connect( m_podcastJob, SIGNAL( result( KIO::Job* ) ), SLOT( fetchResult( KIO::Job* ) ) );
 
 }
 
+void
+PodcastChannel::abortFetch()
+{
+    m_podcastJob->kill();
+
+    stopAnimation();
+    m_title.isEmpty() ?
+        setText( 0, m_url.prettyURL() ) :
+        setText( 0, m_title );
+}
+
 
 void
 PodcastChannel::fetchResult( KIO::Job* job ) //SLOT
@@ -1054,8 +1247,9 @@ PodcastChannel::fetchResult( KIO::Job* job ) //SLOT
         amaroK::StatusBar::instance()->shortMessage( i18n( "Unable to connect to Podcast server." ) );
         debug() << "Unable to retrieve podcast information. KIO Error: " << job->error() << endl;
 
-        if( !m_title )
-            setText( 0, m_url.prettyURL() );
+        m_title.isEmpty() ?
+            setText( 0, m_url.prettyURL() ) :
+            setText( 0, m_title );
 
         setPixmap( 0, SmallIcon("cancel") );
 
@@ -1063,20 +1257,14 @@ PodcastChannel::fetchResult( KIO::Job* job ) //SLOT
     }
 
     KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>( job );
-    QString xml = QString( storedJob->data() );
-
-    // FIXME: Get a better way to detect this
-    if ( xml.contains( "encoding=\"UTF-8\"", false  ) ) {
-         xml = QString::fromUtf8( storedJob->data().data(), storedJob->data().size() );
-    }
 
     QDomDocument d;
 
-    if( !d.setContent( xml ) )
+    if( !d.setContent( storedJob->data() ) )
     {
         amaroK::StatusBar::instance()->shortMessage( i18n("Podcast returned invalid data.") );
 
-        if( !m_title )
+        if( m_title.isEmpty() )
             setText( 0, m_url.prettyURL() );
 
         setPixmap( 0, SmallIcon("cancel") );
@@ -1085,17 +1273,27 @@ PodcastChannel::fetchResult( KIO::Job* job ) //SLOT
     QDomNode type = d.namedItem("rss");
     if( type.isNull() || type.toElement().attribute( "version" ) != "2.0" )
     {
-        amaroK::StatusBar::instance()->shortMessage( i18n("Sorry, only RSS 2.0 feeds for podcasts!") );
+        type = d.namedItem("feed");
+        if( type.isNull() )
+        {
+            //FIXME: Update error message after string freeze
+            amaroK::StatusBar::instance()->shortMessage( i18n("Sorry, only RSS 2.0 feeds for podcasts!") );
 
-        if( !m_title )
-            setText( 0, m_url.prettyURL() );
+            if( m_title.isEmpty() )
+                setText( 0, m_url.prettyURL() );
 
-        setPixmap( 0, SmallIcon("cancel") );
-        return;
+            setPixmap( 0, SmallIcon("cancel") );
+            return;
+        }
+        // feed is ATOM
+        else
+        {
+            setXml( type, ATOM );
+        }
     }
-
-
-    setXml( type.namedItem("channel") );
+    // feed is rss 2.0
+    else
+        setXml( type.namedItem("channel"), RSS );
 
     ///BEGIN Cache the xml
     QFile file( amaroK::saveLocation( "podcasts/" ) +  m_cache );
@@ -1131,6 +1329,71 @@ PodcastChannel::rescan()
     fetch();
 }
 
+void
+PodcastChannel::setSettings( const QString &save, const bool autoFetch, const int fetchType,
+                             const bool purgeItems, const int purgeCount )
+{
+    m_purgeItems = purgeItems;
+    if( m_purgeItems )
+    {
+        m_purgeItems = purgeItems;
+        if( purgeCount < m_purgeCount )
+        {
+            m_purgeCount = purgeCount;
+            purge();
+        }
+    }
+    m_purgeCount = purgeCount;
+
+    /**
+     * Rewrite local url
+     * Move any downloaded media to the new location
+     */
+    if( save != m_saveLocation.path() )
+    {
+        KURL::List copyList;
+        m_saveLocation = KURL::fromPathOrURL( save );
+
+        PodcastItem *item = static_cast<PodcastItem*>( firstChild() );
+        // get a list of the urls of already downloaded items
+        while( item )
+        {
+            if( item->hasDownloaded() )
+            {
+                copyList << item->localUrl();
+                item->setLocalUrlBase( save );
+            }
+
+            item = static_cast<PodcastItem*>( item->nextSibling() );
+        }
+        // move the items
+        if( !copyList.isEmpty() )
+        {
+            KIO::CopyJob* m_podcastMoveJob = new KIO::CopyJob( copyList, m_saveLocation, KIO::CopyJob::Move, false, false );
+            amaroK::StatusBar::instance()->newProgressOperation( m_podcastMoveJob )
+                    .setDescription( i18n( "Moving Podcasts" ) );
+        }
+    }
+
+    if( m_autoScan != autoFetch )
+    {
+        m_autoScan = autoFetch;
+        if( m_autoScan )
+            PlaylistBrowser::instance()->m_podcastItemsToScan.append( this );
+        else
+            PlaylistBrowser::instance()->m_podcastItemsToScan.remove( this );
+    }
+
+    if( m_mediaFetch != fetchType )
+    {
+        m_mediaFetch = fetchType;
+        if( fetchType == AUTOMATIC )
+        {
+            downloadChildren();
+        }
+    }
+}
+
 void
 PodcastChannel::showAbout()
 {
@@ -1164,76 +1427,110 @@ PodcastChannel::setNew( bool n )
 
 /// DONT TOUCH m_url!!!  The podcast has no mention to the location of the xml file, idiots.
 void
-PodcastChannel::setXml( QDomNode xml )
+PodcastChannel::setXml( const QDomNode &xml, const int feedType )
 {
+    const bool isAtom = ( feedType == ATOM );
+
     m_title = xml.namedItem( "title" ).toElement().text();
     setText( 0, m_title );
 
     m_cache = m_title;
     m_cache.replace( " ", "_" );
+    m_cache.replace( "/", "_" );
     m_cache += "_" + m_url.fileName();
 
-    QString weblink = xml.namedItem( "link" ).toElement().text();
+    QString weblink = QString::null;
+    if( isAtom )
+    {
+        weblink = xml.namedItem( "link" ).toElement().attribute( "rel" );
+    }
+    else
+        weblink = xml.namedItem( "link" ).toElement().text();
+
     m_link = KURL::fromPathOrURL( weblink );
 
     m_description = xml.namedItem( "description" ).toElement().text();
     m_copyright   = xml.namedItem( "copyright" ).toElement().text();
 
     PodcastItem *updatingLast = 0;
-    PodcastItem *nextFirst = 0;
 
-    QDomNode n = xml.namedItem( "item" );
+    PodcastItem *first = (PodcastItem*)firstChild();
+
+    QDomNode n;
+    if( isAtom )
+    {
+        n = xml.namedItem( "entry" );
+    }
+    else
+        n = xml.namedItem( "item" );
+
     int  children = 0;
-    bool downloadMedia = ( m_mediaFetch == AVAILABLE );
+    bool downloadMedia = ( m_mediaFetch == AUTOMATIC );
     for( ; !n.isNull(); n = n.nextSibling() )
     {
         if( m_updating )
         {
             // podcasts get inserted in a chronological order,
             // no need to continue traversing, we must have them already
-            if( m_first && m_first->hasXml( n ) ) {
-                if (nextFirst)
-                    m_first = nextFirst;
-                debug() << "breaking" << endl;
+            if( first && first->hasXml( n, feedType ) )
+            {
                 break;
             }
-            if( !n.namedItem( "enclosure" ).toElement().attribute( "url" ).isEmpty() )
+
+            if( isAtom )
             {
-                updatingLast = new PodcastItem( this, updatingLast, n.toElement() );
-
-                if( downloadMedia )
-                    updatingLast->downloadMedia();
-
-                updatingLast->setNew();
-                if (!nextFirst) {
-                    nextFirst = updatingLast;
+                // Atom feeds have multiple nodes called link, only one which has an enclosure.
+                QDomNode nodes = n.namedItem("link");
+                for( ; !nodes.isNull(); nodes = nodes.nextSibling() )
+                {
+                    if( nodes.toElement().attribute("rel") == "enclosure" )
+                    {
+                        updatingLast = new PodcastItem( this, updatingLast, n.toElement(), feedType );
+                        updatingLast->setNew();
+                        break;
+                    }
                 }
             }
+            else if( !n.namedItem( "enclosure" ).toElement().attribute( "url" ).isEmpty() )
+            {
+                updatingLast = new PodcastItem( this, updatingLast, n.toElement(), feedType );
+                updatingLast->setNew();
+            }
         }
         else
         {
-            //only download two episodes initially -> current and 1 back issue
-            //Make configurable?
-            if( children > m_purgeCount )
+            if( m_purgeItems && children > m_purgeCount )
                 break;
 
-            if( !n.namedItem( "enclosure" ).toElement().attribute( "url" ).isEmpty() )
+            if( isAtom )
             {
-                m_last = new PodcastItem( this, m_last, n.toElement() );
-                if( downloadMedia )
-                    m_last->downloadMedia();
-                children++;
-                if (!m_first) {
-                    m_first = m_last;
+                // Atom feeds have multiple nodes called link, only one which has an enclosure.
+                QDomNode nodes = n.namedItem("link");
+                for( ; !nodes.isNull(); nodes = nodes.nextSibling() )
+                {
+                    if( nodes.toElement().attribute("rel") == "enclosure" )
+                    {
+                        updatingLast = new PodcastItem( this, updatingLast, n.toElement(), feedType );
+                        updatingLast->setNew();
+                        break;
+                    }
                 }
             }
+            else if( !n.namedItem( "enclosure" ).toElement().attribute( "url" ).isEmpty() )
+            {
+                m_last = new PodcastItem( this, m_last, n.toElement(), feedType );
+                children++;
+            }
         }
 
     }
 
-    if( childCount() > m_purgeCount )
+    if( m_purgeItems && childCount() > m_purgeCount )
         purge();
 
+    if( downloadMedia )
+        downloadChildren();
+
     if( firstChild() && static_cast<PodcastItem *>( firstChild() )->isNew() && m_updating )
     {
         setNew();
@@ -1246,19 +1543,26 @@ PodcastChannel::setXml( QDomNode xml )
 void
 PodcastChannel::purge()
 {
-    uint removeCount = childCount() - m_purgeCount;
+    int removeCount = childCount() - m_purgeCount;
+    if( removeCount <= 0 )
+        return;
 
-    for( uint i=0; i < removeCount; i++ )
+    KURL::List urls;
+    for( int i=0; i < removeCount; i++ )
     {
         PodcastItem *newLast = 0;
 
         if( m_last && m_last != firstChild() )
             newLast = (PodcastItem *)m_last->itemAbove();
 
+        if( m_last->hasDownloaded() )
+            urls.append( m_last->localUrl() );
 
         delete m_last;
         m_last = newLast;
     }
+    if( !urls.isEmpty() )
+        KIO::del( urls );
 }
 
 QDomElement
@@ -1273,6 +1577,11 @@ PodcastChannel::xml()
         attr.appendChild( t );
         i.appendChild( attr );
 
+        attr = doc.createElement( "savelocation" );
+        t = doc.createTextNode( m_saveLocation.prettyURL() );
+        attr.appendChild( t );
+        i.appendChild( attr );
+
         attr = doc.createElement( "cache" );
         t = doc.createTextNode( m_cache );
         attr.appendChild( t );
@@ -1289,7 +1598,7 @@ PodcastChannel::xml()
         i.appendChild( attr );
 
         attr = doc.createElement( "fetch" );
-        t = doc.createTextNode( ( m_mediaFetch == DOWNLOAD ) ? "download" : "stream" );
+        t = doc.createTextNode( ( m_mediaFetch == AUTOMATIC ) ? "automatic" : "stream" );
         attr.appendChild( t );
         i.appendChild( attr );
 
@@ -1310,7 +1619,7 @@ void
 PodcastChannel::startAnimation()
 {
     if( !m_animationTimer.isActive() )
-        m_animationTimer.start( 100 );
+        m_animationTimer.start( 250 );
 }
 
 void
@@ -1323,20 +1632,21 @@ PodcastChannel::stopAnimation()
 void
 PodcastChannel::slotAnimation()
 {
-    static uint iconCounter = 1;
-
-    iconCounter % 2 ?
+    m_iconCounter % 2 ?
         setPixmap( 0, m_loading1 ):
         setPixmap( 0, m_loading2 );
 
-    iconCounter++;
+    m_iconCounter++;
 }
 
 /////////////////////////////////////////////////////////////////////////////
 ///    CLASS PodcastItem
+///    @note we fucking hate itunes for taking over podcasts and inserting
+///          their own attributes.
 ////////////////////////////////////////////////////////////////////////////
-PodcastItem::PodcastItem( QListViewItem *parent, QListViewItem *after, QDomElement xml )
+PodcastItem::PodcastItem( QListViewItem *parent, QListViewItem *after, const QDomElement &xml, const int feedType )
     : PlaylistBrowserEntry( parent, after )
+      , m_parent( parent )
       , m_localUrl( 0 )
       , m_loading1( QPixmap( locate("data", "amarok/images/loading1.png" ) ) )
       , m_loading2( QPixmap( locate("data", "amarok/images/loading2.png" ) ) )
@@ -1344,25 +1654,47 @@ PodcastItem::PodcastItem( QListViewItem *parent, QListViewItem *after, QDomEleme
       , m_downloaded( false )
       , m_new( false )
 {
+    const bool isAtom = ( feedType == ATOM );
     m_title       = xml.namedItem( "title" ).toElement().text();
 
-    m_description = xml.namedItem( "description" ).toElement().text();
+    if( isAtom )
+    {
+        for( QDomNode n = xml.firstChild(); !n.isNull(); n = n.nextSibling() )
+        {
+            if( n.nodeName() == "summary" )         m_description = n.toElement().text();
+            else if ( n.nodeName() == "author" )    m_author      = n.toElement().text();
+            else if ( n.nodeName() == "published" ) m_date        = n.toElement().text();
+            else if ( n.nodeName() == "link" )
+            {
+                if( n.toElement().attribute( "rel" ) == "enclosure" )
+                {
+                    const QString url = n.toElement().attribute( "href" );
+                    m_url = KURL::fromPathOrURL( url );
+                }
+            }
+        }
+    }
+    else
+    {
+        m_description = xml.namedItem( "description" ).toElement().text();
 
-    if( m_description.isEmpty() )
-        m_description = xml.namedItem( "itunes:summary" ).toElement().text();
+        if( m_description.isEmpty() )
+            m_description = xml.namedItem( "itunes:summary" ).toElement().text();
 
-    m_author      = xml.namedItem( "author" ).toElement().text();
-    m_date        = xml.namedItem( "pubDate" ).toElement().text();
-    m_duration    = xml.namedItem( "enclosure" ).toElement().attribute( "length" ).toInt();
-    m_type        = xml.namedItem( "enclosure" ).toElement().attribute( "type" );
-    const QString url   = xml.namedItem( "enclosure" ).toElement().attribute( "url" );
+        m_author      = xml.namedItem( "author" ).toElement().text();
+        m_date        = xml.namedItem( "pubDate" ).toElement().text();
+        m_duration    = xml.namedItem( "enclosure" ).toElement().attribute( "length" ).toInt();
+        m_type        = xml.namedItem( "enclosure" ).toElement().attribute( "type" );
+        const QString url = xml.namedItem( "enclosure" ).toElement().attribute( "url" );
 
-    m_url         = KURL::fromPathOrURL( url );
+        m_url         = KURL::fromPathOrURL( url );
+    }
 
     if( m_title.isEmpty() )
         m_title = m_url.fileName();
 
-    m_localUrlString = amaroK::saveLocation( "podcasts/data/" );
+    m_localUrlString = dynamic_cast<PodcastChannel*>(m_parent)->saveLocation().path();
+
     QString filename = m_title;
 
     m_localUrlString += filename.replace( " ", "_" ).replace( "/", "_" );;
@@ -1383,10 +1715,14 @@ void
 PodcastItem::downloadMedia()
 {
     if( QFile::exists( m_localUrlString ) )
+    {
+        m_downloaded = true;
         return;
+    }
 
     setText(0, i18n( "Downloading Media..." ) );
 
+    m_iconCounter = 1;
     startAnimation();
     connect( &m_animationTimer, SIGNAL(timeout()), this, SLOT(slotAnimation()) );
     KURL::List list( m_url );
@@ -1403,6 +1739,8 @@ PodcastItem::downloadMedia()
 void
 PodcastItem::abortDownload() //SLOT
 {
+    m_podcastItemJob->kill();
+
     stopAnimation();
     setText( 0, m_title );
     m_downloaded = false;
@@ -1411,6 +1749,8 @@ PodcastItem::abortDownload() //SLOT
 void
 PodcastItem::downloadResult( KIO::Job* job ) //SLOT
 {
+    emit downloadFinished();
+
     stopAnimation();
     setText( 0, m_title );
     if ( !job->error() == 0 ) {
@@ -1427,8 +1767,30 @@ PodcastItem::downloadResult( KIO::Job* job ) //SLOT
 
 
 const bool
-PodcastItem::hasXml( const QDomNode& xml )
+PodcastItem::hasXml( const QDomNode& xml, const int feedType )
 {
+    if( feedType == ATOM )
+    {
+        bool same = true;
+        for( QDomNode n = xml.firstChild(); !n.isNull(); n = n.nextSibling() )
+        {
+            if( n.nodeName() == "summary" )         same &= ( m_description == n.toElement().text() );
+            else if ( n.nodeName() == "author" )    same &= ( m_author      == n.toElement().text() );
+            else if ( n.nodeName() == "published" ) same &= ( m_date        == n.toElement().text() );
+            else if ( n.nodeName() == "link" )
+            {
+                if( n.toElement().attribute( "rel" ) == "enclosure" )
+                {
+                    const QString url = n.toElement().attribute( "href" );
+                    same &= ( m_url.prettyURL() == url );
+                }
+            }
+            if( !same )
+                break;
+        }
+        return same;
+    }
+    //rss
     bool a = m_title           == xml.namedItem( "title" ).toElement().text();
     bool b = m_author          == xml.namedItem( "author" ).toElement().text();
     bool c = m_date            == xml.namedItem( "pubDate" ).toElement().text();
@@ -1439,6 +1801,14 @@ PodcastItem::hasXml( const QDomNode& xml )
     return a && b && c && d && e && f;
 }
 
+void
+PodcastItem::setLocalUrlBase( const QString &s )
+{
+    QString filename = m_localUrl.filename();
+    QString newL = s + filename;
+    m_localUrl = KURL::fromPathOrURL( newL );
+}
+
 void
 PodcastItem::setNew( bool n )
 {
@@ -1472,7 +1842,7 @@ void
 PodcastItem::startAnimation()
 {
     if( !m_animationTimer.isActive() )
-        m_animationTimer.start( 100 );
+        m_animationTimer.start( 250 );
 }
 
 void
@@ -1485,13 +1855,11 @@ PodcastItem::stopAnimation()
 void
 PodcastItem::slotAnimation()
 {
-    static uint iconCounter = 1;
-
-    iconCounter % 2 ?
+    m_iconCounter % 2 ?
         setPixmap( 0, m_loading1 ):
         setPixmap( 0, m_loading2 );
 
-    iconCounter++;
+    m_iconCounter++;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1500,7 +1868,7 @@ PodcastItem::slotAnimation()
 
 SmartPlaylist::SmartPlaylist( QListViewItem *parent, QListViewItem *after, const QString &name, const QString &query )
         : PlaylistBrowserEntry( parent, after, name )
-        , sqlForTags( query )
+        , m_sqlForTags( query )
         , m_title( name )
         , m_dynamic( false )
 {
@@ -1512,7 +1880,7 @@ SmartPlaylist::SmartPlaylist( QListViewItem *parent, QListViewItem *after, const
 
 SmartPlaylist::SmartPlaylist( QListViewItem *parent, QListViewItem *after, const QString &name, const QString &urls, const QString &tags )
         : PlaylistBrowserEntry( parent, after, name )
-        , sqlForTags( tags )
+        , m_sqlForTags( tags )
         , m_title( name )
         , m_dynamic( false )
 {
@@ -1523,21 +1891,21 @@ SmartPlaylist::SmartPlaylist( QListViewItem *parent, QListViewItem *after, const
 }
 
 
-SmartPlaylist::SmartPlaylist( QListViewItem *parent, QListViewItem *after, QDomElement xmlDefinition )
+SmartPlaylist::SmartPlaylist( QListViewItem *parent, QListViewItem *after, const QDomElement &xmlDefinition )
         : PlaylistBrowserEntry( parent, after )
         , m_after ( after )
         , m_dynamic( false )
 {
     setPixmap( 0, SmallIcon( "player_playlist_2" ) );
     setXml( xmlDefinition );
-    setDragEnabled( !sqlForTags.isEmpty() );
+    setDragEnabled( !m_sqlForTags.isEmpty() );
 }
 
-void SmartPlaylist::setXml( QDomElement xml ) {
+void SmartPlaylist::setXml( const QDomElement &xml ) {
     m_xml = xml;
     m_title = xml.attribute( "name" );
     setText( 0, m_title );
-    sqlForTags = xml.namedItem( "sqlquery" ).toElement().text();
+    m_sqlForTags = xml.namedItem( "sqlquery" ).toElement().text();
     static QStringList genres;
     static QStringList artists;
     static QStringList albums;
@@ -1597,6 +1965,13 @@ void SmartPlaylist::setXml( QDomElement xml ) {
 
 }
 
+
+QString SmartPlaylist::query()
+{
+    return m_sqlForTags.replace("(*CurrentTimeT*)" , QString::number(QDateTime::currentDateTime().toTime_t()) );
+}
+
+
 void SmartPlaylist::setDynamic( bool enable )
 {
     if( enable != m_dynamic )
