@@ -104,12 +104,12 @@ void SmartPlaylistEditor::init(QString defaultName)
     makeVBoxMainWidget();
 
     m_fields.clear();
-    m_fields << i18n("Artist") << i18n("Album") << i18n("Genre") << i18n("Title") << i18n("Track #") << i18n("Year")
+    m_fields << i18n("Artist") << i18n("Album") << i18n("Genre") << i18n("Title") << i18n("Length") << i18n("Track #") << i18n("Year")
              << i18n("Comment") << i18n("Play Counter") << i18n("Score") << i18n("First Play") << i18n("Last Play")
              << i18n("Modified Date") << i18n("File Path");
 
     m_dbFields.clear();
-    m_dbFields << "artist.name" << "album.name" << "genre.name" << "tags.title"
+    m_dbFields << "artist.name" << "album.name" << "genre.name" << "tags.title" << "tags.length"
                << "tags.track" << "year.name" << "tags.comment" << "statistics.playcounter"
                << "statistics.percentage" << "statistics.createdate" << "statistics.accessdate"
                << "tags.createdate" << "tags.url";
@@ -264,7 +264,7 @@ QDomElement SmartPlaylistEditor::result() {
         // Iterate through all criteria list
         CriteriaEditor *criteriaeditor = m_criteriaEditorList.first();
         for( int i=0; criteriaeditor; criteriaeditor = m_criteriaEditorList.next(), ++i ) {
-            matches.appendChild( criteriaeditor->getDomSearchCriteria( doc ) );
+            matches.appendChild( doc.importNode( criteriaeditor->getDomSearchCriteria( doc ), true ) );
         }
         if ( m_criteriaEditorList.count() > 1 ) {
             matches.setAttribute( "glue", m_matchCombo->currentItem() == 0 ? "AND" : "OR" );
@@ -306,12 +306,13 @@ void SmartPlaylistEditor::buildQuery()
     QString joins = "tags INNER JOIN year ON year.id=tags.year INNER JOIN genre ON genre.id=tags.genre"
                     " INNER JOIN artist ON artist.id=tags.artist INNER JOIN album ON album.id=tags.album";
     QString whereStr;
+    QString criteriaListStr;
     QString orderStr;
     QString limitStr;
 
     //where expression
     if( m_matchCheck->isChecked() ) {
-        whereStr += " WHERE (";
+        criteriaListStr += " (";
 
         CriteriaEditor *criteria = m_criteriaEditorList.first();
         for( int i=0; criteria; criteria = m_criteriaEditorList.next(), i++ ) {
@@ -331,8 +332,9 @@ void SmartPlaylistEditor::buildQuery()
                 str.prepend( " " + op + " (");
 
             }
-            whereStr += str+")";
+            criteriaListStr += str+")";
         }
+        whereStr = " WHERE" + criteriaListStr;
     }
 
     //order by expression
@@ -384,8 +386,10 @@ void SmartPlaylistEditor::buildQuery()
         if( !joins.contains( table ) ) {
             joins += " INNER JOIN statistics ON statistics.url=tags.url";
         }
-        whereStr = QString("%1 AND %1 = \"(*ExpandString*)\"").arg(whereStr).arg(field);
-
+        if ( !criteriaListStr.isEmpty() )
+            whereStr = QString(" WHERE (%1) AND %2 = '(*ExpandString*)'").arg(criteriaListStr).arg(field);
+        else
+            whereStr = QString("WHERE %1 = '(*ExpandString*)'").arg(field);
         m_expandQuery = "SELECT album.name, artist.name, genre.name, tags.title, year.name, "
                             "tags.comment, tags.track, tags.bitrate, tags.length, tags.samplerate, tags.url"
                             " FROM " + joins + whereStr + orderStr + limitStr + ";";
@@ -490,9 +494,13 @@ CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QD
                         m_dateCombo->setCurrentItem(2);
                 }
                 else {
-                    m_dateEdit1->setDate( QDate::fromString( values.first() ) );
-                    if( condition == i18n("is between") )
-                        m_dateEdit2->setDate( QDate::fromString( values.last() ) );
+                    QDateTime dt;
+                    dt.setTime_t( values.first().toUInt() );
+                    m_dateEdit1->setDate( dt.date() );
+                    if( condition == i18n("is between") ) {
+                        dt.setTime_t( values.last().toUInt() );
+                        m_dateEdit2->setDate( dt.date() );
+                    }
                 }
                 break;
             }
@@ -591,7 +599,7 @@ QString CriteriaEditor::getSearchCriteria()
                 else if( m_dateCombo->currentItem() == 1 ) //months
                     time=86400*30*n;
                 else time=86400*365*n; //years
-                value += QString("strftime('%s','now') - %1 AND strftime('%s','now')").arg( time );
+                value += "(*CurrentTimeT*)" + QString(" - %1 AND ").arg(time) + "(*CurrentTimeT*)";
             }
             else {
                 QDateTime datetime1( m_dateEdit1->date() );
@@ -610,16 +618,16 @@ QString CriteriaEditor::getSearchCriteria()
 
 
     if( criteria == i18n("contains") )
-        searchCriteria += " LIKE \"%" + value + "%\"";
+        searchCriteria += " LIKE '%" + value + "%'";
     else if( criteria == i18n("does not contain") )
-        searchCriteria += " NOT LIKE \"%" + value + "%\"";
+        searchCriteria += " NOT LIKE '%" + value + "%'";
     else if( criteria == i18n("is") ) {
         if( m_currentValueType == Date )
             searchCriteria += " BETWEEN ";
         else
             searchCriteria += " = ";
         if( m_currentValueType == String || m_currentValueType == AutoCompletionString )
-            value.prepend("\"").append("\"");
+            value.prepend("'").append("'");
         searchCriteria += value;
         if (field=="statistics.playcounter" && value=="0") {
             searchCriteria += " OR statistics.playcounter IS NULL";
@@ -631,13 +639,13 @@ QString CriteriaEditor::getSearchCriteria()
         else
             searchCriteria += " <> ";
         if( m_currentValueType == String || m_currentValueType == AutoCompletionString )
-            value.prepend("\"").append("\"");
+            value.prepend("'").append("'");
         searchCriteria += value;
     }
     else if( criteria == i18n("starts with") )
-        searchCriteria += " LIKE \"" + value + "%\"";
+        searchCriteria += " LIKE '" + value + "%'";
     else if( criteria == i18n("ends with") )
-        searchCriteria += " LIKE \"%" + value + "\"";
+        searchCriteria += " LIKE '%" + value + "'";
     else if( criteria == i18n("is greater than") || criteria == i18n("is after") )
         searchCriteria += " > " + value;
     else if( criteria == i18n("is smaller than") || criteria == i18n("is before" ) )
@@ -854,16 +862,17 @@ int CriteriaEditor::getValueType( int index )
             valueType = AutoCompletionString;
             break;
         case 3:
-        case 6:
-        case 12:
+        case 7:
+        case 13:
             valueType = String;
             break;
         case 4:
-        case 7:
+        case 5:
         case 8:
+        case 9:
             valueType = Number;
             break;
-        case 5:
+        case 6:
             valueType = Year;
             break;
         default: valueType = Date;
