@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
  ***************************************************************************/
 
 
@@ -55,6 +55,7 @@ namespace KDE
         void showCloseButton( const bool show );
         void showCounter( const bool show );
         void setImage( const QString &location );
+        void setImage( const QPixmap &pix );
         void setMaskEffect( const MaskEffect type ) { m_maskEffect = type; }
         void setText( const QString &text );
         void setTimeout( const int time ) { m_timeout = time; }
@@ -63,10 +64,8 @@ namespace KDE
         void close();
         void display();
 
-    private slots:
-        void timerEvent( QTimerEvent* );
-
     protected:
+        void timerEvent( QTimerEvent* );
         void countDown();
 
         /**
