@@ -1,12 +1,15 @@
 // (c) 2004 Mark Kretschmann <markey@web.de>
 // (c) 2004 Pierpaolo Di Panfilo <pippo_dp@libero.it>
+// (c) 2005 Alexandre Pereira de Oliveira <aleprj@gmail.com>
 // See COPYING file for licensing information.
 
-#include "collectiondb.h"
 #include "debug.h"
+#include "collectionbrowser.h"
+#include "collectiondb.h"
 #include "metabundle.h"
 #include "playlist.h"
 #include "playlistitem.h"
+#include "statusbar.h"       //for status messages
 #include "tagdialog.h"
 #include "trackpickerdialog.h"
 
@@ -14,6 +17,7 @@
 #include <taglib/tag.h>
 #include <taglib/tfile.h>
 #include <taglib/tstring.h>
+#include <taglib/id3v2framefactory.h>
 
 #include <qfile.h>
 #include <qlabel.h>
@@ -24,6 +28,7 @@
 #include <kcombobox.h>
 #include <kcursor.h>
 #include <kglobal.h>
+#include <kiconloader.h>
 #include <klineedit.h>
 #include <kmessagebox.h>
 #include <knuminput.h>
@@ -34,6 +39,8 @@
 TagDialog::TagDialog( const KURL& url, QWidget* parent )
     : TagDialogBase( parent )
     , m_bundle( MetaBundle( url ) )
+    , m_score ( CollectionDB::instance()->getSongPercentage( url.path() ) )
+    , m_playcount( CollectionDB::instance()->getPlayCount( url.path() ) )
     , m_playlistItem( 0 )
     , m_currentCover( 0 )
 {
@@ -44,11 +51,12 @@ TagDialog::TagDialog( const KURL& url, QWidget* parent )
 TagDialog::TagDialog( const KURL::List list, QWidget* parent )
     : TagDialogBase( parent )
     , m_bundle( MetaBundle() )
+    , m_score ( 0 )
     , m_playlistItem( 0 )
     , m_urlList( list )
     , m_currentCover( 0 )
 {
-    setCaption( kapp->makeStdCaption( i18n( "Meta Information (Multiple Files)" ) ) );
+    setCaption( kapp->makeStdCaption( i18n("1 Track", "Information for %n Tracks", list.count()) ) );
     init();
 }
 
@@ -56,6 +64,8 @@ TagDialog::TagDialog( const KURL::List list, QWidget* parent )
 TagDialog::TagDialog( const MetaBundle& mb, PlaylistItem* item, QWidget* parent )
     : TagDialogBase( parent )
     , m_bundle( mb )
+    , m_score ( CollectionDB::instance()->getSongPercentage( mb.url().path() ) )
+    , m_playcount( CollectionDB::instance()->getPlayCount( mb.url().path() ) )
     , m_playlistItem( item )
     , m_currentCover( 0 )
 {
@@ -100,14 +110,21 @@ TagDialog::previousTrack()
 {
     if( !m_playlistItem->itemAbove() ) return;
 
-    if( hasChanged() )
-        storeTags();
+    storeTags();
 
     m_playlistItem = (PlaylistItem *)m_playlistItem->itemAbove();
-    QMap<QString, MetaBundle>::ConstIterator it;
-    it = storedTags.find( m_playlistItem->url().path() );
-    m_bundle = it != storedTags.end() ? it.data() : MetaBundle( m_playlistItem );
 
+    QMap<QString, MetaBundle>::ConstIterator itTags;
+    itTags = storedTags.find( m_playlistItem->url().path() );
+    m_bundle = itTags != storedTags.end() ? itTags.data() : MetaBundle( m_playlistItem );
+
+    QMap<QString, int>::ConstIterator itScores;
+    itScores = storedScores.find( m_playlistItem->url().path() );
+    m_score = itScores != storedScores.end()
+        ? itScores.data()
+        : CollectionDB::instance()->getSongPercentage( m_playlistItem->url().path() );
+
+    m_playcount = CollectionDB::instance()->getPlayCount( m_playlistItem->url().path() );
     readTags();
 }
 
@@ -117,13 +134,20 @@ TagDialog::nextTrack()
 {
     if( !m_playlistItem->itemBelow() ) return;
 
-    if( hasChanged() )
-        storeTags();
+    storeTags();
 
     m_playlistItem = (PlaylistItem *)m_playlistItem->itemBelow();
-    QMap<QString, MetaBundle>::ConstIterator it;
-    it = storedTags.find( m_playlistItem->url().path() );
-    m_bundle = it != storedTags.end() ? it.data() : MetaBundle( m_playlistItem );
+
+    QMap<QString, MetaBundle>::ConstIterator itTags;
+    itTags = storedTags.find( m_playlistItem->url().path() );
+    m_bundle = itTags != storedTags.end() ? itTags.data() : MetaBundle( m_playlistItem );
+
+    QMap<QString, int>::ConstIterator itScores;
+    itScores = storedScores.find( m_playlistItem->url().path() );
+    m_score = itScores != storedScores.end()
+        ? itScores.data()
+        : CollectionDB::instance()->getSongPercentage( m_playlistItem->url().path() );
+    m_playcount = CollectionDB::instance()->getPlayCount( m_playlistItem->url().path() );
 
     readTags();
 }
@@ -132,7 +156,7 @@ TagDialog::nextTrack()
 inline void
 TagDialog::checkModified() //SLOT
 {
-    pushButton_ok->setEnabled( hasChanged() );
+    pushButton_ok->setEnabled( hasChanged() || storedTags.count() > 0 || storedScores.count() > 0 );
 }
 
 
@@ -235,12 +259,14 @@ void TagDialog::init()
     // the UI file due to bug in Designer
     kIntSpinBox_track->setSpecialValueText( " " );
     kIntSpinBox_year->setSpecialValueText( " " );
+    kIntSpinBox_score->setSpecialValueText( " " );
 
     //HACK due to deficiency in Qt that will be addressed in version 4
     // QSpinBox doesn't emit valueChanged if you edit the value with
     // the lineEdit until you change the keyboard focus
-    connect( kIntSpinBox_year->child( "qt_spinbox_edit" ), SIGNAL(textChanged( const QString& )), SLOT(checkModified()) );
+    connect( kIntSpinBox_year->child( "qt_spinbox_edit" ),  SIGNAL(textChanged( const QString& )), SLOT(checkModified()) );
     connect( kIntSpinBox_track->child( "qt_spinbox_edit" ), SIGNAL(textChanged( const QString& )), SLOT(checkModified()) );
+    connect( kIntSpinBox_score->child( "qt_spinbox_edit" ), SIGNAL(textChanged( const QString& )), SLOT(checkModified()) );
 
 
     // Connects for modification check
@@ -253,6 +279,7 @@ void TagDialog::init()
     connect( kComboBox_genre,  SIGNAL(textChanged( const QString& )), SLOT(checkModified()) );
     connect( kIntSpinBox_track,SIGNAL(valueChanged( int )),           SLOT(checkModified()) );
     connect( kIntSpinBox_year, SIGNAL(valueChanged( int )),           SLOT(checkModified()) );
+    connect( kIntSpinBox_score,SIGNAL(valueChanged( int )),           SLOT(checkModified()) );
     connect( kLineEdit_comment,SIGNAL(textChanged( const QString& )), SLOT(checkModified()) );
 
     // Remember original button text
@@ -265,7 +292,7 @@ void TagDialog::init()
     connect( pushButton_next,     SIGNAL(clicked()), SLOT(nextTrack()) );
 
     // draw an icon onto the open-in-konqui button
-    pushButton_open->setPixmap( QPixmap( locate( "data", QString( "amarok/images/folder_crystal.png" ) ), "PNG" ) );
+     pushButton_open->setIconSet( kapp->iconLoader()->loadIconSet( "fileopen", KIcon::Small ) );
 
     if( !m_playlistItem ) {
         pushButton_previous->hide();
@@ -291,17 +318,21 @@ void TagDialog::init()
 
 void TagDialog::readTags()
 {
+    bool local = m_bundle.url().isLocalFile();
+
     kLineEdit_title->setText( m_bundle.title() );
     kComboBox_artist->setCurrentText( m_bundle.artist() );
     kComboBox_album->setCurrentText( m_bundle.album() );
     kComboBox_genre->setCurrentText( m_bundle.genre() );
     kIntSpinBox_track->setValue( m_bundle.track().toInt() );
     kIntSpinBox_year->setValue( m_bundle.year().toInt() );
+    kIntSpinBox_score->setValue( m_score );
     kLineEdit_comment->setText( m_bundle.comment() );
     kLineEdit_length->setText( m_bundle.prettyLength() );
+    kLineEdit_playcount->setText( QString::number( m_playcount ) );
     kLineEdit_bitrate->setText( m_bundle.prettyBitrate() );
     kLineEdit_samplerate->setText( m_bundle.prettySampleRate() );
-    kLineEdit_location->setText( m_bundle.url().isLocalFile() ? m_bundle.url().path() : m_bundle.url().url() );
+    kLineEdit_location->setText( local ? m_bundle.url().path() : m_bundle.url().url() );
     // draw the album cover on the dialog
     QString cover = CollectionDB::instance()->albumImage( m_bundle );
 
@@ -310,25 +341,23 @@ void TagDialog::readTags()
         m_currentCover = cover;
     }
 
-    // Disable the tag editor for streams
-    if ( !m_bundle.url().isLocalFile() )
-    {
-          kLineEdit_title->setReadOnly( true );
-          kComboBox_artist->setEnabled( false );
-          kComboBox_album->setEnabled( false );
-          kComboBox_genre->setEnabled( false );
-          kIntSpinBox_track->setEnabled( false );
-          kIntSpinBox_year->setEnabled( false );
-          kLineEdit_comment->setEnabled( false );
-    }
+    // enable only for local files
+    kLineEdit_title->setReadOnly( !local );
+    kComboBox_artist->setEnabled( local );
+    kComboBox_album->setEnabled( local );
+    kComboBox_genre->setEnabled( local );
+    kIntSpinBox_track->setEnabled( local );
+    kIntSpinBox_year->setEnabled( local );
+    kIntSpinBox_score->setEnabled( local );
+    kLineEdit_comment->setEnabled( local );
 
     // If it's a local file, write the directory to m_path, else disable the "open in konqui" button
-    if ( m_bundle.url().isLocalFile() )
+    if ( local )
         m_path = m_bundle.url().directory();
     else
         pushButton_open->setEnabled( false );
 
-    pushButton_ok->setEnabled( storedTags.count() > 0 );
+    pushButton_ok->setEnabled( storedTags.count() > 0 || storedScores.count() > 0 );
 
 #if HAVE_TUNEPIMP
     pushButton_musicbrainz->setEnabled( m_bundle.url().isLocalFile() );
@@ -358,6 +387,8 @@ TagDialog::setMultipleTracksMode()
     line1->hide();
     lengthLabel->hide();
     kLineEdit_length->hide();
+    playcountLabel->hide();
+    kLineEdit_playcount->hide();
     bitrateLabel->hide();
     kLineEdit_bitrate->hide();
     samplerateLabel->hide();
@@ -376,13 +407,14 @@ TagDialog::readMultipleTracks()
     const KURL::List::ConstIterator end = m_urlList.end();
     KURL::List::ConstIterator it = m_urlList.begin();
     MetaBundle first( *it );
-    bool artist=true, album=true, genre=true, comment=true, year=true;
-    
+    bool artist=true, album=true, genre=true, comment=true, year=true, score=true;
+    uint scoreFirst = CollectionDB::instance()->getSongPercentage( first.url().path() );
+
     for ( ; it != end; ++it ) {
         if( !(*it).isLocalFile() ) {
             // If we have a non local file, don't even lose more time comparing, just leave
-            artist=false; album=false; genre=false; comment=false, year=false;
-            break; 
+            artist=false; album=false; genre=false; comment=false, year=false, score=false;
+            break;
         }
         MetaBundle mb( *it );
         if ( artist && mb.artist()!=first.artist() ) { artist=false; };
@@ -390,8 +422,12 @@ TagDialog::readMultipleTracks()
         if ( genre && mb.genre()!=first.genre() ) { genre=false; };
         if ( comment && mb.comment()!=first.comment() ) { comment=false; };
         if ( year && mb.year()!=first.year() ) { year=false; };
-        
-        if (!artist && !album && !genre && !comment && !year)
+
+        uint scoreCurrent = CollectionDB::instance()->getSongPercentage( mb.url().path() );
+        if ( score && scoreFirst != scoreCurrent )
+            score = false;
+
+        if (!artist && !album && !genre && !comment && !year && !score)
             break;
     }
     // Set them in the dialog and in m_bundle ( so we don't break hasChanged() )
@@ -415,6 +451,10 @@ TagDialog::readMultipleTracks()
         m_bundle.setYear( first.year() );
         kIntSpinBox_year->setValue( first.year().toInt() );
     }
+    if (score) {
+        m_score = scoreFirst;
+        kIntSpinBox_score->setValue( scoreFirst );
+    }
 }
 
 inline bool
@@ -426,42 +466,60 @@ equalString( const QString &a, const QString &b )
 bool
 TagDialog::hasChanged()
 {
+    return changes();
+}
+
+int
+TagDialog::changes()
+{
+    int result=TagDialog::NOCHANGE;
     bool modified = false;
     modified |= !equalString( kComboBox_artist->lineEdit()->text(), m_bundle.artist() );
     modified |= !equalString( kComboBox_album->lineEdit()->text(), m_bundle.album() );
     modified |= !equalString( kComboBox_genre->lineEdit()->text(), m_bundle.genre() );
     modified |= kIntSpinBox_year->value()  != m_bundle.year().toInt();
+
     modified |= !equalString( kLineEdit_comment->text(), m_bundle.comment() );
     if (!m_urlList.count()) { //ignore these on MultipleTracksMode
         modified |= !equalString( kLineEdit_title->text(), m_bundle.title() );
         modified |= kIntSpinBox_track->value() != m_bundle.track().toInt();
     }
+    if (modified)
+        result |= TagDialog::TAGSCHANGED;
 
-    return modified;
+    if (kIntSpinBox_score->value() != m_score)
+        result |= TagDialog::SCORECHANGED;
+
+    return result;
 }
 
 void
 TagDialog::storeTags()
 {
-    MetaBundle mb;
+    int result = changes();
     QString url = m_bundle.url().path();
+    if( result & TagDialog::TAGSCHANGED ) {
+        MetaBundle mb;
 
-    mb.setPath( url );
-    mb.setTitle( kLineEdit_title->text() );
-    mb.setArtist( kComboBox_artist->currentText() );
-    mb.setAlbum( kComboBox_album->currentText() );
-    mb.setComment( kLineEdit_comment->text() );
-    mb.setGenre( kComboBox_genre->currentText() );
-    mb.setTrack( QString::number( kIntSpinBox_track->value() ) );
-    mb.setYear( QString::number( kIntSpinBox_year->value() ) );
-    mb.setLength( m_bundle.length() );
-    mb.setBitrate( m_bundle.bitrate() );
-    mb.setSampleRate( m_bundle.sampleRate() );
+        mb.setPath( url );
+        mb.setTitle( kLineEdit_title->text() );
+        mb.setArtist( kComboBox_artist->currentText() );
+        mb.setAlbum( kComboBox_album->currentText() );
+        mb.setComment( kLineEdit_comment->text() );
+        mb.setGenre( kComboBox_genre->currentText() );
+        mb.setTrack( QString::number( kIntSpinBox_track->value() ) );
+        mb.setYear( QString::number( kIntSpinBox_year->value() ) );
+        mb.setLength( m_bundle.length() );
+        mb.setBitrate( m_bundle.bitrate() );
+        mb.setSampleRate( m_bundle.sampleRate() );
+
+        storedTags.insert( url, mb );
+    }
+    if( result & TagDialog::SCORECHANGED )
+        storedScores.insert( url, kIntSpinBox_score->value() );
 
-    storedTags.insert( url, mb );
 }
 
-
 void
 TagDialog::saveTags()
 {
@@ -469,13 +527,19 @@ TagDialog::saveTags()
         saveMultipleTracks();
     else
     {
-        if( hasChanged() )
-            storeTags();
 
-        QMap<QString, MetaBundle>::ConstIterator it;
-        for( it = storedTags.begin(); it != storedTags.end(); ++it ) {
+        storeTags();
+        QMap<QString, MetaBundle>::ConstIterator endStore( storedTags.end() );
+        for(QMap<QString, MetaBundle>::ConstIterator it = storedTags.begin(); it != endStore; ++it ) {
             if( writeTag( it.data(), it == --storedTags.end() ) )    //update the collection browser if it's the last track
                 Playlist::instance()->updateMetaData( it.data() );
+            else
+                amaroK::StatusBar::instance()->longMessage( i18n(
+                    "Sorry, the tag for %1 could not be changed." ).arg( it.data().prettyURL() ), KDE::StatusBar::Error );
+        }
+        QMap<QString, int>::ConstIterator end( storedScores.end() );
+        for(QMap<QString, int>::ConstIterator it = storedScores.begin(); it != end; ++it ) {
+            CollectionDB::instance()->setSongPercentage( it.key(), it.data() );
         }
     }
 }
@@ -484,40 +548,79 @@ TagDialog::saveTags()
 void
 TagDialog::saveMultipleTracks()
 {
+    bool tagsChanged = false, anyTrack = false;
     const KURL::List::ConstIterator end = m_urlList.end();
     for ( KURL::List::ConstIterator it = m_urlList.begin(); it != end; ++it ) {
         if( !(*it).isLocalFile() ) continue;
 
         MetaBundle mb( *it );
 
-        if( !kComboBox_artist->currentText().isEmpty() )
+   /* we have to update the values if they changed, so:
+   1) !kLineEdit_field->text().isEmpty() && kLineEdit_field->text() != mb.field
+          i.e.: The user wrote something on the field, and it's different from
+          what we have in the tag.
+   2) !m_bundle.field().isEmpty() && kLineEdit_field->text().isEmpty()
+          i.e.: The user was shown some value for the field (it was the same
+          for all selected tracks), and he deliberately emptied it.
+  TODO: All this mess is because the dialog uses "" to represent what the user
+        doesn't want to change, maybe we can think of something better?
+   */
+        if( !kComboBox_artist->currentText().isEmpty() && kComboBox_artist->currentText() != mb.artist() ||
+             kComboBox_artist->currentText().isEmpty() && !m_bundle.artist().isEmpty() ) {
             mb.setArtist( kComboBox_artist->currentText() );
+            tagsChanged = true;
+        }
 
-        if( !kComboBox_album->currentText().isEmpty() )
+        if( !kComboBox_album->currentText().isEmpty() && kComboBox_album->currentText() != mb.album() ||
+             kComboBox_album->currentText().isEmpty() && !m_bundle.album().isEmpty() ) {
             mb.setAlbum( kComboBox_album->currentText() );
-
-        if( !kComboBox_genre->currentText().isEmpty() )
+            tagsChanged = true;
+        }
+        if( !kComboBox_genre->currentText().isEmpty() && kComboBox_genre->currentText() != mb.genre() ||
+             kComboBox_genre->currentText().isEmpty() && !m_bundle.genre().isEmpty() ) {
             mb.setGenre( kComboBox_genre->currentText() );
-
-        if( !kLineEdit_comment->text().isEmpty() )
+            tagsChanged = true;
+        }
+        if( !kLineEdit_comment->text().isEmpty() && kLineEdit_comment->text() != mb.comment() ||
+             kLineEdit_comment->text().isEmpty() && !m_bundle.comment().isEmpty() ) {
             mb.setComment( kLineEdit_comment->text() );
-
-        if( kIntSpinBox_year->value() )
+            tagsChanged = true;
+        }
+        if( kIntSpinBox_year->value() && kIntSpinBox_year->value() != mb.year().toInt() ||
+            !kIntSpinBox_year->value() && m_bundle.year().toInt() ) {
             mb.setYear( QString::number( kIntSpinBox_year->value() ) );
+            tagsChanged = true;
+        }
+        if( kIntSpinBox_score->value() && kIntSpinBox_score->value() != m_score ||
+            !kIntSpinBox_score->value() && m_score )
+            CollectionDB::instance()->setSongPercentage( mb.url().path(), kIntSpinBox_score->value() );
 
-        if( writeTag( mb, it == --m_urlList.end() ) )    //update the collection browser if it's the last track
-            Playlist::instance()->updateMetaData( mb );
+        if ( tagsChanged ) { // We only try to update the file if one of the tags changed
+            if( writeTag( mb, false ) ) //leave the Collection Browser update to be made after all changes
+                Playlist::instance()->updateMetaData( mb );
+            else
+                amaroK::StatusBar::instance()->longMessage( i18n(
+                    "Sorry, the tag for %1 could not be changed." ).arg( mb.prettyURL() ), KDE::StatusBar::Error );
+        }
+        anyTrack |= tagsChanged;
     }
+    // if any change was really made for any of the tracks, update Collection Browser.
+    if (anyTrack)
+        CollectionView::instance()->renderView();
 }
 
 
 bool
 TagDialog::writeTag( MetaBundle mb, bool updateCB )
 {
-    QCString path = QFile::encodeName( mb.url().path() );
+    //Set default codec to UTF-8 (see bugs 111246 and 111232)
+    TagLib::ID3v2::FrameFactory::instance()->setDefaultTextEncoding(TagLib::String::UTF8);
 
+    QCString path = QFile::encodeName( mb.url().path() );
     if ( !TagLib::File::isWritable( path ) ) {
-        KMessageBox::sorry( this, i18n( "TagLib claims this file is not writable." ) );
+        amaroK::StatusBar::instance()->longMessage( i18n(
+           "TagLib claims %1 file is not writable." ).arg( path ), KDE::StatusBar::Error );
+
         return false;
     }
 
@@ -540,7 +643,7 @@ TagDialog::writeTag( MetaBundle mb, bool updateCB )
         bool result = f.save();
         if( result )
             //update the collection db
-            CollectionDB::instance()->updateTags( path, mb, updateCB );
+            CollectionDB::instance()->updateTags( mb.url().path(), mb, updateCB );
 
         QApplication::restoreOverrideCursor();
 
